# -*- coding: utf-8 -*-

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.proxy as proxy

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, call, patch
from tests.support.unit import TestCase


class ProxyTestCase(TestCase, LoaderModuleMockMixin):
    """
        Validate the proxy state
    """

    def setup_loader_modules(self):
        return {proxy: {}}

    def test_set_proxy_macos(self):
        """
            Test to make sure we can set the proxy settings on macOS
        """
        with patch.dict(proxy.__grains__, {"os": "Darwin"}):
            expected = {
                "changes": {
                    "new": [
                        {
                            "port": "3128",
                            "server": "192.168.0.1",
                            "service": "http",
                            "user": "frank",
                        },
                        {
                            "port": "3128",
                            "server": "192.168.0.1",
                            "service": "https",
                            "user": "frank",
                        },
                        {
                            "port": "3128",
                            "server": "192.168.0.1",
                            "service": "ftp",
                            "user": "frank",
                        },
                        {"bypass_domains": ["salt.com", "test.com"]},
                    ]
                },
                "comment": "http proxy settings updated correctly\nhttps proxy settings updated correctly\nftp proxy "
                "settings updated correctly\nProxy bypass domains updated correctly\n",
                "name": "192.168.0.1",
                "result": True,
            }

            set_proxy_mock = MagicMock(return_value=True)
            patches = {
                "proxy.get_http_proxy": MagicMock(return_value={}),
                "proxy.get_https_proxy": MagicMock(return_value={}),
                "proxy.get_ftp_proxy": MagicMock(return_value={}),
                "proxy.get_proxy_bypass": MagicMock(return_value=[]),
                "proxy.set_http_proxy": set_proxy_mock,
                "proxy.set_https_proxy": set_proxy_mock,
                "proxy.set_ftp_proxy": set_proxy_mock,
                "proxy.set_proxy_bypass": set_proxy_mock,
            }

            with patch.dict(proxy.__salt__, patches):
                out = proxy.managed(
                    "192.168.0.1",
                    "3128",
                    user="frank",
                    password="passw0rd",
                    bypass_domains=["salt.com", "test.com"],
                )
                out["changes"]["new"][-1]["bypass_domains"] = sorted(
                    out["changes"]["new"][-1]["bypass_domains"]
                )

                calls = [
                    call("192.168.0.1", "3128", "frank", "passw0rd", "Ethernet"),
                    call("192.168.0.1", "3128", "frank", "passw0rd", "Ethernet"),
                    call("192.168.0.1", "3128", "frank", "passw0rd", "Ethernet"),
                    call(["salt.com", "test.com"], "Ethernet"),
                ]

                set_proxy_mock.assert_has_calls(calls)
                self.assertEqual(out, expected)

    def test_set_proxy_macos_same(self):
        """
            Test to make sure we can set the proxy settings on macOS
        """
        with patch.dict(proxy.__grains__, {"os": "Darwin"}):
            expected = {
                "changes": {},
                "comment": "http proxy settings already set.\nhttps proxy settings already set.\nftp proxy settings"
                " already set.\nProxy bypass domains are already set correctly.\n",
                "name": "192.168.0.1",
                "result": True,
            }

            proxy_val = {"enabled": True, "server": "192.168.0.1", "port": "3128"}

            set_proxy_mock = MagicMock()
            patches = {
                "proxy.get_http_proxy": MagicMock(return_value=proxy_val),
                "proxy.get_https_proxy": MagicMock(return_value=proxy_val),
                "proxy.get_ftp_proxy": MagicMock(return_value=proxy_val),
                "proxy.get_proxy_bypass": MagicMock(
                    return_value=["test.com", "salt.com"]
                ),
                "proxy.set_http_proxy": set_proxy_mock,
                "proxy.set_https_proxy": set_proxy_mock,
                "proxy.set_ftp_proxy": set_proxy_mock,
                "proxy.set_proxy_bypass": set_proxy_mock,
            }

            with patch.dict(proxy.__salt__, patches):
                out = proxy.managed(
                    "192.168.0.1",
                    "3128",
                    user="frank",
                    password="passw0rd",
                    bypass_domains=["salt.com", "test.com"],
                )

                assert not set_proxy_mock.called
                self.assertEqual(out, expected)

    def test_set_proxy_windows(self):
        """
            Test to make sure we can set the proxy settings on Windows
        """
        with patch.dict(proxy.__grains__, {"os": "Windows"}):
            expected = {
                "changes": {},
                "comment": "Proxy settings updated correctly",
                "name": "192.168.0.1",
                "result": True,
            }

            set_proxy_mock = MagicMock(return_value=True)
            patches = {
                "proxy.get_proxy_win": MagicMock(return_value={}),
                "proxy.get_proxy_bypass": MagicMock(return_value=[]),
                "proxy.set_proxy_win": set_proxy_mock,
            }

            with patch.dict(proxy.__salt__, patches):
                out = proxy.managed(
                    "192.168.0.1",
                    "3128",
                    user="frank",
                    password="passw0rd",
                    bypass_domains=["salt.com", "test.com"],
                )

                set_proxy_mock.assert_called_once_with(
                    "192.168.0.1",
                    "3128",
                    ["http", "https", "ftp"],
                    ["salt.com", "test.com"],
                )
                self.assertEqual(out, expected)

    def test_set_proxy_windows_same(self):
        """
            Test to make sure we can set the proxy settings on Windows
        """
        with patch.dict(proxy.__grains__, {"os": "Windows"}):
            expected = {
                "changes": {},
                "comment": "Proxy settings already correct.",
                "name": "192.168.0.1",
                "result": True,
            }

            proxy_val = {
                "enabled": True,
                "http": {"enabled": True, "server": "192.168.0.1", "port": "3128"},
                "https": {"enabled": True, "server": "192.168.0.1", "port": "3128"},
                "ftp": {"enabled": True, "server": "192.168.0.1", "port": "3128"},
            }

            set_proxy_mock = MagicMock(return_value=True)
            patches = {
                "proxy.get_proxy_win": MagicMock(return_value=proxy_val),
                "proxy.get_proxy_bypass": MagicMock(
                    return_value=["salt.com", "test.com"]
                ),
                "proxy.set_proxy_win": set_proxy_mock,
            }

            with patch.dict(proxy.__salt__, patches):
                out = proxy.managed(
                    "192.168.0.1",
                    "3128",
                    user="frank",
                    password="passw0rd",
                    bypass_domains=["salt.com", "test.com"],
                )

                assert not set_proxy_mock.called
                self.assertEqual(out, expected)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Rahul Handay <rahulha@saltstack.com>
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.xmpp as xmpp

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class XmppTestCase(TestCase, LoaderModuleMockMixin):
    """
        Validate the xmpp state
    """

    def setup_loader_modules(self):
        return {xmpp: {}}

    def test_send_msg(self):
        """
            Test to send a message to an XMPP user
        """
        ret = {"name": "salt", "changes": {}, "result": None, "comment": ""}
        with patch.dict(xmpp.__opts__, {"test": True}):
            ret.update({"comment": "Need to send message to myaccount: salt"})
            self.assertDictEqual(
                xmpp.send_msg("salt", "myaccount", "salt@saltstack.com"), ret
            )

        with patch.dict(xmpp.__opts__, {"test": False}):
            mock = MagicMock(return_value=True)
            with patch.dict(
                xmpp.__salt__, {"xmpp.send_msg": mock, "xmpp.send_msg_multi": mock}
            ):
                ret.update(
                    {"result": True, "comment": "Sent message to myaccount: salt"}
                )
                self.assertDictEqual(
                    xmpp.send_msg("salt", "myaccount", "salt@saltstack.com"), ret
                )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.mysql_user as mysql_user
import salt.utils.data

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class MysqlUserTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.mysql_user
    """

    def setup_loader_modules(self):
        return {mysql_user: {}}

    # 'present' function tests: 1

    def test_present(self):
        """
        Test to ensure that the named user is present with
         the specified properties.
        """
        name = "frank"
        password = "bob@cat"

        ret = {"name": name, "result": False, "comment": "", "changes": {}}

        mock = MagicMock(
            side_effect=[
                True,
                False,
                True,
                False,
                False,
                True,
                False,
                False,
                False,
                False,
                False,
                True,
            ]
        )
        mock_t = MagicMock(return_value=True)
        mock_f = MagicMock(return_value=False)
        mock_str = MagicMock(return_value="salt")
        mock_none = MagicMock(return_value=None)
        mock_sn = MagicMock(side_effect=[None, "salt", None, None, None])
        with patch.object(salt.utils.data, "is_true", mock_f):
            comt = (
                "Either password or password_hash must be specified,"
                " unless allow_passwordless is True"
            )
            ret.update({"comment": comt})
            self.assertDictEqual(mysql_user.present(name), ret)

        with patch.dict(
            mysql_user.__salt__,
            {"mysql.user_exists": mock, "mysql.user_chpass": mock_t},
        ):
            with patch.object(salt.utils.data, "is_true", mock_t):
                comt = (
                    "User frank@localhost is already present" " with passwordless login"
                )
                ret.update({"comment": comt, "result": True})
                self.assertDictEqual(mysql_user.present(name), ret)

                with patch.object(mysql_user, "_get_mysql_error", mock_str):
                    ret.update({"comment": "salt", "result": False})
                    self.assertDictEqual(mysql_user.present(name), ret)

            with patch.object(mysql_user, "_get_mysql_error", mock_str):
                comt = (
                    "User frank@localhost is already present"
                    " with the desired password"
                )
                ret.update({"comment": comt, "result": True})
                self.assertDictEqual(mysql_user.present(name, password=password), ret)

                with patch.object(mysql_user, "_get_mysql_error", mock_str):
                    ret.update({"comment": "salt", "result": False})
                    self.assertDictEqual(
                        mysql_user.present(name, password=password), ret
                    )

                with patch.object(mysql_user, "_get_mysql_error", mock_none):
                    with patch.dict(mysql_user.__opts__, {"test": True}):
                        comt = (
                            "Password for user frank@localhost" " is set to be changed"
                        )
                        ret.update({"comment": comt, "result": None})
                        self.assertDictEqual(
                            mysql_user.present(name, password=password), ret
                        )

                with patch.object(mysql_user, "_get_mysql_error", mock_sn):
                    with patch.dict(mysql_user.__opts__, {"test": False}):
                        ret.update({"comment": "salt", "result": False})
                        self.assertDictEqual(
                            mysql_user.present(name, password=password), ret
                        )

                    with patch.dict(mysql_user.__opts__, {"test": True}):
                        comt = "User frank@localhost is set to be added"
                        ret.update({"comment": comt, "result": None})
                        self.assertDictEqual(
                            mysql_user.present(name, password=password), ret
                        )

                    with patch.dict(mysql_user.__opts__, {"test": False}):
                        comt = "Password for user frank@localhost" " has been changed"
                        ret.update(
                            {
                                "comment": comt,
                                "result": True,
                                "changes": {name: "Updated"},
                            }
                        )
                        self.assertDictEqual(
                            mysql_user.present(name, password=password), ret
                        )

    # 'absent' function tests: 1

    def test_absent(self):
        """
        Test to ensure that the named user is absent.
        """
        name = "frank_exampledb"

        ret = {"name": name, "result": True, "comment": "", "changes": {}}

        mock = MagicMock(side_effect=[True, True, True, False, False, False])
        mock_t = MagicMock(side_effect=[True, False])
        mock_str = MagicMock(return_value="salt")
        mock_none = MagicMock(return_value=None)
        with patch.dict(
            mysql_user.__salt__,
            {"mysql.user_exists": mock, "mysql.user_remove": mock_t},
        ):
            with patch.dict(mysql_user.__opts__, {"test": True}):
                comt = "User frank_exampledb@localhost is set to be removed"
                ret.update({"comment": comt, "result": None})
                self.assertDictEqual(mysql_user.absent(name), ret)

            with patch.dict(mysql_user.__opts__, {"test": False}):
                comt = "User frank_exampledb@localhost has been removed"
                ret.update(
                    {
                        "comment": comt,
                        "result": True,
                        "changes": {"frank_exampledb": "Absent"},
                    }
                )
                self.assertDictEqual(mysql_user.absent(name), ret)

                with patch.object(mysql_user, "_get_mysql_error", mock_str):
                    comt = "User frank_exampledb@localhost has been removed"
                    ret.update({"comment": "salt", "result": False, "changes": {}})
                    self.assertDictEqual(mysql_user.absent(name), ret)

                    comt = "User frank_exampledb@localhost has been removed"
                    ret.update({"comment": "salt"})
                    self.assertDictEqual(mysql_user.absent(name), ret)

                with patch.object(mysql_user, "_get_mysql_error", mock_none):
                    comt = (
                        "User frank_exampledb@localhost is not present,"
                        " so it cannot be removed"
                    )
                    ret.update({"comment": comt, "result": True, "changes": {}})
                    self.assertDictEqual(mysql_user.absent(name), ret)



# -*- coding: utf-8 -*-

from __future__ import absolute_import

import pytest
import salt.modules.chocolatey as choco
import salt.utils.platform
from tests.support.case import ModuleCase
from tests.support.helpers import destructiveTest
from tests.support.unit import skipIf


@skipIf(not salt.utils.platform.is_windows(), "Tests for only Windows")
@pytest.mark.windows_whitelisted
class ChocolateyModuleTest(ModuleCase):
    """
    Validate Chocolatey module
    """

    @destructiveTest
    def setUp(self):
        """
        Ensure that Chocolatey is installed
        """
        self._chocolatey_bin = choco._find_chocolatey()
        if "ERROR" in self._chocolatey_bin:
            #    self.fail("Chocolatey is not installed")
            self.run_function("chocolatey.bootstrap")
        super(ChocolateyModuleTest, self).setUp()

    def test_list_(self):
        ret = self.run_function("chocolatey.list", narrow="adobereader", exact=True)
        self.assertTrue("adobereader" in ret)



# -*- coding: utf-8 -*-
"""
Tests for salt.utils.iam
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import salt.utils.iam as iam
from salt.ext import six
from tests.support.unit import TestCase, skipIf


class IamTestCase(TestCase):
    @skipIf(six.PY3, "Only needs to run on Python 2")
    def test__convert_key_to_str(self):
        """
        Makes sure that unicode string is converted to a str type on PY2
        """
        key = "foo"
        expected = key.encode("utf-8")
        self.assertEqual(iam._convert_key_to_str(key), expected)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Nicole Thomas <nicole@saltstack.com>
"""

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.modules.mac_brew_pkg as mac_brew
import salt.utils.pkg
from salt.exceptions import CommandExecutionError

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, Mock, patch
from tests.support.unit import TestCase

TAPS_STRING = "homebrew/dupes\nhomebrew/science\nhomebrew/x11"
TAPS_LIST = ["homebrew/dupes", "homebrew/science", "homebrew/x11"]
HOMEBREW_BIN = "/usr/local/bin/brew"


class BrewTestCase(TestCase, LoaderModuleMockMixin):
    """
    TestCase for salt.modules.mac_brew module
    """

    def setup_loader_modules(self):
        return {mac_brew: {"__opts__": {"user": MagicMock(return_value="bar")}}}

    # '_list_taps' function tests: 1

    def test_list_taps(self):
        """
        Tests the return of the list of taps
        """
        mock_taps = MagicMock(return_value={"stdout": TAPS_STRING, "retcode": 0})
        mock_user = MagicMock(return_value="foo")
        mock_cmd = MagicMock(return_value="")
        with patch.dict(
            mac_brew.__salt__,
            {"file.get_user": mock_user, "cmd.run_all": mock_taps, "cmd.run": mock_cmd},
        ):
            self.assertEqual(mac_brew._list_taps(), TAPS_LIST)

    # '_tap' function tests: 3

    def test_tap_installed(self):
        """
        Tests if tap argument is already installed or not
        """
        with patch(
            "salt.modules.mac_brew_pkg._list_taps", MagicMock(return_value=TAPS_LIST)
        ):
            self.assertTrue(mac_brew._tap("homebrew/science"))

    def test_tap_failure(self):
        """
        Tests if the tap installation failed
        """
        mock_failure = MagicMock(
            return_value={"stdout": "", "stderr": "", "retcode": 1}
        )
        mock_user = MagicMock(return_value="foo")
        mock_cmd = MagicMock(return_value="")
        with patch.dict(
            mac_brew.__salt__,
            {
                "cmd.run_all": mock_failure,
                "file.get_user": mock_user,
                "cmd.run": mock_cmd,
            },
        ), patch("salt.modules.mac_brew_pkg._list_taps", MagicMock(return_value={})):
            self.assertFalse(mac_brew._tap("homebrew/test"))

    def test_tap(self):
        """
        Tests adding unofficial GitHub repos to the list of brew taps
        """
        mock_failure = MagicMock(return_value={"retcode": 0})
        mock_user = MagicMock(return_value="foo")
        mock_cmd = MagicMock(return_value="")
        with patch.dict(
            mac_brew.__salt__,
            {
                "cmd.run_all": mock_failure,
                "file.get_user": mock_user,
                "cmd.run": mock_cmd,
            },
        ), patch(
            "salt.modules.mac_brew_pkg._list_taps", MagicMock(return_value=TAPS_LIST)
        ):
            self.assertTrue(mac_brew._tap("homebrew/test"))

    # '_homebrew_bin' function tests: 1

    def test_homebrew_bin(self):
        """
        Tests the path to the homebrew binary
        """
        mock_path = MagicMock(return_value="/usr/local")
        with patch.dict(mac_brew.__salt__, {"cmd.run": mock_path}):
            self.assertEqual(mac_brew._homebrew_bin(), "/usr/local/bin/brew")

    # 'list_pkgs' function tests: 2
    # Only tested a few basics
    # Full functionality should be tested in integration phase

    def test_list_pkgs_removed(self):
        """
        Tests removed implementation
        """
        self.assertEqual(mac_brew.list_pkgs(removed=True), {})

    def test_list_pkgs_versions_true(self):
        """
        Tests if pkg.list_pkgs is already in context and is a list
        """
        mock_context = {"foo": ["bar"]}
        with patch.dict(mac_brew.__context__, {"pkg.list_pkgs": mock_context}):
            self.assertEqual(mac_brew.list_pkgs(versions_as_list=True), mock_context)

    def test_list_pkgs_homebrew_cask_pakages(self):
        """
        Tests if pkg.list_pkgs list properly homebrew cask packages
        """

        def custom_call_brew(cmd, failhard=True):
            result = dict()
            if cmd == "info --json=v1 --installed":
                result = {
                    "stdout": '[{"name":"zsh","full_name":"zsh","oldname":null,'
                    '"aliases":[],"homepage":"https://www.zsh.org/",'
                    '"versions":{"stable":"5.7.1","devel":null,"head":"HEAD","bottle":true},'
                    '"installed":[{"version":"5.7.1","used_options":[],'
                    '"built_as_bottle":true,"poured_from_bottle":true,'
                    '"runtime_dependencies":[{"full_name":"ncurses","version":"6.1"},'
                    '{"full_name":"pcre","version":"8.42"}],'
                    '"installed_as_dependency":false,"installed_on_request":true}]}]',
                    "stderr": "",
                    "retcode": 0,
                }
            elif cmd == "cask list --versions":
                result = {
                    "stdout": "macvim 8.1.151\nfont-firacode-nerd-font 2.0.0",
                    "stderr": "",
                    "retcode": 0,
                }
            elif cmd == "cask info macvim":
                result = {
                    "stdout": "macvim: 8.1.1517,156 (auto_updates)\n"
                    "https://github.com/macvim-dev/macvim\n"
                    "/usr/local/Caskroom/macvim/8.1.151 (64B)\n"
                    "From: https://github.com/Homebrew/homebrew-cask/blob/master/Casks/macvim.rb\n"
                    "==> Name\n"
                    "MacVim",
                    "stderr": "",
                    "retcode": 0,
                }
            elif cmd == "cask info font-firacode-nerd-font":
                result = {
                    "stdout": "font-firacode-nerd-font: 2.0.0\n"
                    "https://github.com/ryanoasis/nerd-fonts\n"
                    "/usr/local/Caskroom/font-firacode-nerd-font/2.0.0 (35 files, 64.8MB)\n"
                    "From: https://github.com/Homebrew/homebrew-cask-fonts/blob/master/Casks/font-firacode-nerd-font.rb\n"
                    "==> Name\n"
                    "FuraCode Nerd Font (FiraCode)",
                    "stderr": "",
                    "retcode": "",
                }

            return result

        def custom_add_pkg(ret, name, newest_version):
            ret[name] = newest_version
            return ret

        expected_pkgs = {
            "zsh": "5.7.1",
            "homebrew/cask/macvim": "8.1.151",
            "homebrew/cask-fonts/font-firacode-nerd-font": "2.0.0",
        }

        with patch(
            "salt.modules.mac_brew_pkg._call_brew", custom_call_brew
        ), patch.dict(
            mac_brew.__salt__,
            {
                "pkg_resource.add_pkg": custom_add_pkg,
                "pkg_resource.sort_pkglist": MagicMock(),
            },
        ):
            self.assertEqual(mac_brew.list_pkgs(versions_as_list=True), expected_pkgs)

    # 'version' function tests: 1

    def test_version(self):
        """
        Tests version name returned
        """
        mock_version = MagicMock(return_value="0.1.5")
        with patch.dict(mac_brew.__salt__, {"pkg_resource.version": mock_version}):
            self.assertEqual(mac_brew.version("foo"), "0.1.5")

    # 'latest_version' function tests: 0
    # It has not been fully implemented

    # 'remove' function tests: 1
    # Only tested a few basics
    # Full functionality should be tested in integration phase

    def test_remove(self):
        """
        Tests if package to be removed exists
        """
        mock_params = MagicMock(return_value=({"foo": None}, "repository"))
        with patch(
            "salt.modules.mac_brew_pkg.list_pkgs", return_value={"test": "0.1.5"}
        ), patch.dict(mac_brew.__salt__, {"pkg_resource.parse_targets": mock_params}):
            self.assertEqual(mac_brew.remove("foo"), {})

    # 'refresh_db' function tests: 2

    def test_refresh_db_failure(self):
        """
        Tests an update of homebrew package repository failure
        """
        mock_user = MagicMock(return_value="foo")
        mock_failure = MagicMock(
            return_value={"stdout": "", "stderr": "", "retcode": 1}
        )
        with patch.dict(
            mac_brew.__salt__, {"file.get_user": mock_user, "cmd.run_all": mock_failure}
        ), patch(
            "salt.modules.mac_brew_pkg._homebrew_bin",
            MagicMock(return_value=HOMEBREW_BIN),
        ):
            with patch.object(salt.utils.pkg, "clear_rtag", Mock()):
                self.assertRaises(CommandExecutionError, mac_brew.refresh_db)

    def test_refresh_db(self):
        """
        Tests a successful update of homebrew package repository
        """
        mock_user = MagicMock(return_value="foo")
        mock_success = MagicMock(return_value={"retcode": 0})
        with patch.dict(
            mac_brew.__salt__, {"file.get_user": mock_user, "cmd.run_all": mock_success}
        ), patch(
            "salt.modules.mac_brew_pkg._homebrew_bin",
            MagicMock(return_value=HOMEBREW_BIN),
        ):
            with patch.object(salt.utils.pkg, "clear_rtag", Mock()):
                self.assertTrue(mac_brew.refresh_db())

    # 'install' function tests: 1
    # Only tested a few basics
    # Full functionality should be tested in integration phase

    def test_install(self):
        """
        Tests if package to be installed exists
        """
        mock_params = MagicMock(return_value=[None, None])
        with patch.dict(mac_brew.__salt__, {"pkg_resource.parse_targets": mock_params}):
            self.assertEqual(mac_brew.install("name=foo"), {})

    # "hold" function tests: 2
    # Only tested a few basics
    # Full functionality should be tested in integration phase

    def test_hold(self):
        """
        Tests holding if package is installed
        """
        mock_user = MagicMock(return_value="foo")
        mock_cmd = MagicMock(return_value="")
        mock_cmd_all = MagicMock(
            return_value={"pid": 12345, "retcode": 0, "stderr": "", "stdout": ""}
        )
        _expected = {
            "foo": {
                "changes": {"new": "hold", "old": "install"},
                "comment": "Package foo is now being held.",
                "name": "foo",
                "result": True,
            }
        }

        mock_params = MagicMock(return_value=({"foo": None}, "repository"))
        with patch(
            "salt.modules.mac_brew_pkg.list_pkgs", return_value={"foo": "0.1.5"}
        ), patch.dict(
            mac_brew.__salt__,
            {
                "file.get_user": mock_user,
                "pkg_resource.parse_targets": mock_params,
                "cmd.run_all": mock_cmd_all,
                "cmd.run": mock_cmd,
            },
        ):
            self.assertEqual(mac_brew.hold("foo"), _expected)

    def test_hold_not_installed(self):
        """
        Tests holding if package is not installed
        """
        mock_user = MagicMock(return_value="foo")
        mock_cmd = MagicMock(return_value="")
        mock_cmd_all = MagicMock(
            return_value={"pid": 12345, "retcode": 0, "stderr": "", "stdout": ""}
        )
        _expected = {
            "foo": {
                "changes": {},
                "comment": "Package foo does not have a state.",
                "name": "foo",
                "result": False,
            }
        }

        mock_params = MagicMock(return_value=({"foo": None}, "repository"))
        with patch("salt.modules.mac_brew_pkg.list_pkgs", return_value={}), patch.dict(
            mac_brew.__salt__,
            {
                "file.get_user": mock_user,
                "pkg_resource.parse_targets": mock_params,
                "cmd.run_all": mock_cmd_all,
                "cmd.run": mock_cmd,
            },
        ):
            self.assertEqual(mac_brew.hold("foo"), _expected)

    def test_hold_pinned(self):
        """
        Tests holding if package is already pinned
        """
        mock_user = MagicMock(return_value="foo")
        mock_cmd = MagicMock(return_value="")
        mock_cmd_all = MagicMock(
            return_value={"pid": 12345, "retcode": 0, "stderr": "", "stdout": ""}
        )
        _expected = {
            "foo": {
                "changes": {},
                "comment": "Package foo is already set to be held.",
                "name": "foo",
                "result": True,
            }
        }

        mock_params = MagicMock(return_value=({"foo": None}, "repository"))
        with patch(
            "salt.modules.mac_brew_pkg.list_pkgs", return_value={"foo": "0.1.5"}
        ), patch(
            "salt.modules.mac_brew_pkg._list_pinned", return_value=["foo"]
        ), patch.dict(
            mac_brew.__salt__,
            {
                "file.get_user": mock_user,
                "pkg_resource.parse_targets": mock_params,
                "cmd.run_all": mock_cmd_all,
                "cmd.run": mock_cmd,
            },
        ):
            self.assertEqual(mac_brew.hold("foo"), _expected)

    # "unhold" function tests: 2
    # Only tested a few basics
    # Full functionality should be tested in integration phase

    def test_unhold(self):
        """
        Tests unholding if package is installed
        """
        mock_user = MagicMock(return_value="foo")
        mock_cmd = MagicMock(return_value="")
        mock_cmd_all = MagicMock(
            return_value={"pid": 12345, "retcode": 0, "stderr": "", "stdout": ""}
        )
        _expected = {
            "foo": {
                "changes": {"new": "install", "old": "hold"},
                "comment": "Package foo is no longer being held.",
                "name": "foo",
                "result": True,
            }
        }

        mock_params = MagicMock(return_value=({"foo": None}, "repository"))
        with patch(
            "salt.modules.mac_brew_pkg.list_pkgs", return_value={"foo": "0.1.5"}
        ), patch(
            "salt.modules.mac_brew_pkg._list_pinned", return_value=["foo"]
        ), patch.dict(
            mac_brew.__salt__,
            {
                "file.get_user": mock_user,
                "pkg_resource.parse_targets": mock_params,
                "cmd.run_all": mock_cmd_all,
                "cmd.run": mock_cmd,
            },
        ):
            self.assertEqual(mac_brew.unhold("foo"), _expected)

    def test_unhold_not_installed(self):
        """
        Tests unholding if package is not installed
        """
        mock_user = MagicMock(return_value="foo")
        mock_cmd = MagicMock(return_value="")
        mock_cmd_all = MagicMock(
            return_value={"pid": 12345, "retcode": 0, "stderr": "", "stdout": ""}
        )
        _expected = {
            "foo": {
                "changes": {},
                "comment": "Package foo does not have a state.",
                "name": "foo",
                "result": False,
            }
        }

        mock_params = MagicMock(return_value=({"foo": None}, "repository"))
        with patch("salt.modules.mac_brew_pkg.list_pkgs", return_value={}), patch(
            "salt.modules.mac_brew_pkg._list_pinned", return_value=["foo"]
        ), patch.dict(
            mac_brew.__salt__,
            {
                "file.get_user": mock_user,
                "pkg_resource.parse_targets": mock_params,
                "cmd.run_all": mock_cmd_all,
                "cmd.run": mock_cmd,
            },
        ):
            self.assertEqual(mac_brew.unhold("foo"), _expected)

    def test_unhold_not_pinned(self):
        """
        Tests unholding if package is not installed
        """
        mock_user = MagicMock(return_value="foo")
        mock_cmd = MagicMock(return_value="")
        mock_cmd_all = MagicMock(
            return_value={"pid": 12345, "retcode": 0, "stderr": "", "stdout": ""}
        )
        _expected = {
            "foo": {
                "changes": {},
                "comment": "Package foo is already set not to be held.",
                "name": "foo",
                "result": True,
            }
        }

        mock_params = MagicMock(return_value=({"foo": None}, "repository"))
        with patch(
            "salt.modules.mac_brew_pkg.list_pkgs", return_value={"foo": "0.1.5"}
        ), patch("salt.modules.mac_brew_pkg._list_pinned", return_value=[]), patch.dict(
            mac_brew.__salt__,
            {
                "file.get_user": mock_user,
                "pkg_resource.parse_targets": mock_params,
                "cmd.run_all": mock_cmd_all,
                "cmd.run": mock_cmd,
            },
        ):
            self.assertEqual(mac_brew.unhold("foo"), _expected)



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.status as status

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class StatusTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.status
    """

    def setup_loader_modules(self):
        return {status: {}}

    # 'loadavg' function tests: 1
    def test_loadavg(self):
        """
        Test to return the current load average for the specified minion.
        """
        name = "mymonitor"

        ret = {"name": name, "changes": {}, "result": True, "data": {}, "comment": ""}

        mock = MagicMock(return_value=[])
        with patch.dict(status.__salt__, {"status.loadavg": mock}):
            comt = "Requested load average mymonitor not available "
            ret.update({"comment": comt, "result": False})
            self.assertDictEqual(status.loadavg(name), ret)

        mock = MagicMock(return_value={name: 3})
        with patch.dict(status.__salt__, {"status.loadavg": mock}):
            comt = "Min must be less than max"
            ret.update({"comment": comt, "result": False})
            self.assertDictEqual(status.loadavg(name, 1, 5), ret)

            comt = "Load avg is below minimum of 4 at 3.0"
            ret.update({"comment": comt, "data": 3})
            self.assertDictEqual(status.loadavg(name, 5, 4), ret)

            comt = "Load avg above maximum of 2 at 3.0"
            ret.update({"comment": comt, "data": 3})
            self.assertDictEqual(status.loadavg(name, 2, 1), ret)

            comt = "Load avg in acceptable range"
            ret.update({"comment": comt, "result": True})
            self.assertDictEqual(status.loadavg(name, 3, 1), ret)

    # 'process' function tests: 1

    def test_process(self):
        """
        Test to return whether the specified signature
        is found in the process tree.
        """
        name = "mymonitor"

        ret = {"name": name, "changes": {}, "result": True, "data": {}, "comment": ""}

        mock = MagicMock(side_effect=[{}, {name: 1}])
        with patch.dict(status.__salt__, {"status.pid": mock}):
            comt = 'Process signature "mymonitor" not found '
            ret.update({"comment": comt, "result": False})
            self.assertDictEqual(status.process(name), ret)

            comt = 'Process signature "mymonitor" was found '
            ret.update({"comment": comt, "result": True, "data": {name: 1}})
            self.assertDictEqual(status.process(name), ret)



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

import copy
import os

import salt.modules.grains as grainsmod
import salt.utils.dictupdate as dictupdate

# Import Salt libs
from salt.exceptions import SaltException

# Import 3rd-party libs
from salt.utils.odict import OrderedDict
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch

# Import Salt Testing libs
from tests.support.runtests import RUNTIME_VARS
from tests.support.unit import TestCase


class GrainsModuleTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        conf_file = os.path.join(RUNTIME_VARS.TMP, "__salt_test_grains")
        cachedir = os.path.join(RUNTIME_VARS.TMP, "__salt_test_grains_cache_dir")
        if not os.path.isdir(cachedir):
            os.makedirs(cachedir)
        return {
            grainsmod: {
                "__opts__": {"conf_file": conf_file, "cachedir": cachedir},
                "__salt__": {"saltutil.refresh_grains": MagicMock()},
            }
        }

    def test_filter_by(self):
        with patch.dict(
            grainsmod.__grains__,
            {"os_family": "MockedOS", "1": "1", "2": "2", "roles": ["A", "B"]},
        ):

            dict1 = {"A": "B", "C": {"D": {"E": "F", "G": "H"}}}
            dict2 = {
                "default": {"A": "B", "C": {"D": "E"}},
                "1": {"A": "X"},
                "2": {"C": {"D": "H"}},
                "MockedOS": {"A": "Z"},
            }

            mdict1 = {"D": {"E": "I"}, "J": "K"}
            mdict2 = {"A": "Z"}
            mdict3 = {"C": {"D": "J"}}

            # test None result with non existent grain and no default
            res = grainsmod.filter_by(dict1, grain="xxx")
            self.assertIs(res, None)

            # test None result with os_family grain and no matching result
            res = grainsmod.filter_by(dict1)
            self.assertIs(res, None)

            # test with non existent grain, and a given default key
            res = grainsmod.filter_by(dict1, grain="xxx", default="C")
            self.assertEqual(res, {"D": {"E": "F", "G": "H"}})

            # add a merge dictionary, F disappears
            res = grainsmod.filter_by(dict1, grain="xxx", merge=mdict1, default="C")
            self.assertEqual(res, {"D": {"E": "I", "G": "H"}, "J": "K"})
            # dict1 was altered, reestablish
            dict1 = {"A": "B", "C": {"D": {"E": "F", "G": "H"}}}

            # default is not present in dict1, check we only have merge in result
            res = grainsmod.filter_by(dict1, grain="xxx", merge=mdict1, default="Z")
            self.assertEqual(res, mdict1)

            # default is not present in dict1, and no merge, should get None
            res = grainsmod.filter_by(dict1, grain="xxx", default="Z")
            self.assertIs(res, None)

            # test giving a list as merge argument raise exception
            self.assertRaises(
                SaltException, grainsmod.filter_by, dict1, "xxx", ["foo"], "C"
            )

            # Now, re-test with an existing grain (os_family), but with no match.
            res = grainsmod.filter_by(dict1)
            self.assertIs(res, None)
            res = grainsmod.filter_by(dict1, default="C")
            self.assertEqual(res, {"D": {"E": "F", "G": "H"}})
            res = grainsmod.filter_by(dict1, merge=mdict1, default="C")
            self.assertEqual(res, {"D": {"E": "I", "G": "H"}, "J": "K"})
            # dict1 was altered, reestablish
            dict1 = {"A": "B", "C": {"D": {"E": "F", "G": "H"}}}
            res = grainsmod.filter_by(dict1, merge=mdict1, default="Z")
            self.assertEqual(res, mdict1)
            res = grainsmod.filter_by(dict1, default="Z")
            self.assertIs(res, None)
            # this one is in fact a traceback in updatedict, merging a string with a dictionary
            self.assertRaises(
                TypeError, grainsmod.filter_by, dict1, merge=mdict1, default="A"
            )

            # Now, re-test with a matching grain.
            dict1 = {"A": "B", "MockedOS": {"D": {"E": "F", "G": "H"}}}
            res = grainsmod.filter_by(dict1)
            self.assertEqual(res, {"D": {"E": "F", "G": "H"}})
            res = grainsmod.filter_by(dict1, default="A")
            self.assertEqual(res, {"D": {"E": "F", "G": "H"}})
            res = grainsmod.filter_by(dict1, merge=mdict1, default="A")
            self.assertEqual(res, {"D": {"E": "I", "G": "H"}, "J": "K"})
            # dict1 was altered, reestablish
            dict1 = {"A": "B", "MockedOS": {"D": {"E": "F", "G": "H"}}}
            res = grainsmod.filter_by(dict1, merge=mdict1, default="Z")
            self.assertEqual(res, {"D": {"E": "I", "G": "H"}, "J": "K"})
            # dict1 was altered, reestablish
            dict1 = {"A": "B", "MockedOS": {"D": {"E": "F", "G": "H"}}}
            res = grainsmod.filter_by(dict1, default="Z")
            self.assertEqual(res, {"D": {"E": "F", "G": "H"}})

            # Test when grain value is a list
            dict1 = {"A": "B", "C": {"D": {"E": "F", "G": "H"}}}
            res = grainsmod.filter_by(dict1, grain="roles", default="C")
            self.assertEqual(res, "B")
            # Test default when grain value is a list
            dict1 = {"Z": "B", "C": {"D": {"E": "F", "G": "H"}}}
            res = grainsmod.filter_by(dict1, grain="roles", default="C")
            self.assertEqual(res, {"D": {"E": "F", "G": "H"}})

            # Test with wildcard pattern in the lookup_dict keys
            dict1 = {"*OS": "B", "C": {"D": {"E": "F", "G": "H"}}}
            res = grainsmod.filter_by(dict1)
            self.assertEqual(res, "B")
            # Test with non-strings in lookup_dict keys
            # Issue #38094
            dict1 = {1: 2, 3: {4: 5}, "*OS": "B"}
            res = grainsmod.filter_by(dict1)
            self.assertEqual(res, "B")
            # Test with sequence pattern with roles
            dict1 = {"Z": "B", "[BC]": {"D": {"E": "F", "G": "H"}}}
            res = grainsmod.filter_by(dict1, grain="roles", default="Z")
            self.assertEqual(res, {"D": {"E": "F", "G": "H"}})

            # Base tests
            # NOTE: these may fail to detect errors if dictupdate.update() is broken
            # but then the unit test for dictupdate.update() should fail and expose
            # that.  The purpose of these tests is it validate the logic of how
            # in filter_by() processes its arguments.

            # Test with just the base
            res = grainsmod.filter_by(dict2, grain="xxx", default="xxx", base="default")
            self.assertEqual(res, dict2["default"])

            # Test the base with the OS grain look-up
            res = grainsmod.filter_by(dict2, default="xxx", base="default")
            self.assertEqual(
                res,
                dictupdate.update(copy.deepcopy(dict2["default"]), dict2["MockedOS"]),
            )

            # Test the base with default
            res = grainsmod.filter_by(dict2, grain="xxx", base="default")
            self.assertEqual(res, dict2["default"])

            res = grainsmod.filter_by(dict2, grain="1", base="default")
            self.assertEqual(
                res, dictupdate.update(copy.deepcopy(dict2["default"]), dict2["1"])
            )

            res = grainsmod.filter_by(dict2, base="default", merge=mdict2)
            self.assertEqual(
                res,
                dictupdate.update(
                    dictupdate.update(
                        copy.deepcopy(dict2["default"]), dict2["MockedOS"]
                    ),
                    mdict2,
                ),
            )

            res = grainsmod.filter_by(dict2, base="default", merge=mdict3)
            self.assertEqual(
                res,
                dictupdate.update(
                    dictupdate.update(
                        copy.deepcopy(dict2["default"]), dict2["MockedOS"]
                    ),
                    mdict3,
                ),
            )

    def test_append_not_a_list(self):
        # Failing append to an existing string, without convert
        with patch.dict(grainsmod.__grains__, {"b": "bval"}):
            res = grainsmod.append("b", "d")
            self.assertEqual(res, "The key b is not a valid list")
            self.assertEqual(grainsmod.__grains__, {"b": "bval"})

            # Failing append to an existing dict
            with patch.dict(grainsmod.__grains__, {"b": {"b1": "bval1"}}):
                res = grainsmod.append("b", "d")
                self.assertEqual(res, "The key b is not a valid list")
                self.assertEqual(grainsmod.__grains__, {"b": {"b1": "bval1"}})

    def test_append_already_in_list(self):
        # Append an existing value
        with patch.dict(grainsmod.__grains__, {"a_list": ["a", "b", "c"], "b": "bval"}):
            res = grainsmod.append("a_list", "b")
            self.assertEqual(res, "The val b was already in the list a_list")
            self.assertEqual(
                grainsmod.__grains__, {"a_list": ["a", "b", "c"], "b": "bval"}
            )

    def test_append_ok(self):
        # Append to an existing list
        with patch.dict(grainsmod.__grains__, {"a_list": ["a", "b", "c"], "b": "bval"}):
            res = grainsmod.append("a_list", "d")
            self.assertEqual(res, {"a_list": ["a", "b", "c", "d"]})
            self.assertEqual(
                grainsmod.__grains__, {"a_list": ["a", "b", "c", "d"], "b": "bval"}
            )

        # Append to an non existing list
        with patch.dict(grainsmod.__grains__, {"b": "bval"}):
            res = grainsmod.append("a_list", "d")
            self.assertEqual(res, {"a_list": ["d"]})
            self.assertEqual(grainsmod.__grains__, {"a_list": ["d"], "b": "bval"})

        # Append to an existing string, with convert
        with patch.dict(grainsmod.__grains__, {"b": "bval"}):
            res = grainsmod.append("b", "d", convert=True)
            self.assertEqual(res, {"b": ["bval", "d"]})
            self.assertEqual(grainsmod.__grains__, {"b": ["bval", "d"]})

        # Append to an existing dict, with convert
        with patch.dict(grainsmod.__grains__, {"b": {"b1": "bval1"}}):
            res = grainsmod.append("b", "d", convert=True)
            self.assertEqual(res, {"b": [{"b1": "bval1"}, "d"]})
            self.assertEqual(grainsmod.__grains__, {"b": [{"b1": "bval1"}, "d"]})

    def test_append_nested_not_a_list(self):
        # Failing append to an existing string, without convert
        with patch.dict(grainsmod.__grains__, {"a": {"b": "bval"}}):
            res = grainsmod.append("a:b", "d")
            self.assertEqual(res, "The key a:b is not a valid list")
            self.assertEqual(grainsmod.__grains__, {"a": {"b": "bval"}})

        # Failing append to an existing dict
        with patch.dict(grainsmod.__grains__, {"a": {"b": {"b1": "bval1"}}}):
            res = grainsmod.append("a:b", "d")
            self.assertEqual(res, "The key a:b is not a valid list")
            self.assertEqual(grainsmod.__grains__, {"a": {"b": {"b1": "bval1"}}})

    def test_append_nested_already_in_list(self):
        # Append an existing value
        with patch.dict(
            grainsmod.__grains__, {"a": {"a_list": ["a", "b", "c"], "b": "bval"}}
        ):
            res = grainsmod.append("a:a_list", "b")
            self.assertEqual(res, "The val b was already in the list a:a_list")
            self.assertEqual(
                grainsmod.__grains__, {"a": {"a_list": ["a", "b", "c"], "b": "bval"}}
            )

    def test_append_nested_ok(self):
        # Append to an existing list
        with patch.dict(
            grainsmod.__grains__, {"a": {"a_list": ["a", "b", "c"], "b": "bval"}}
        ):
            res = grainsmod.append("a:a_list", "d")
            self.assertEqual(res, {"a": {"a_list": ["a", "b", "c", "d"], "b": "bval"}})
            self.assertEqual(
                grainsmod.__grains__,
                {"a": {"a_list": ["a", "b", "c", "d"], "b": "bval"}},
            )

        # Append to an non existing list
        with patch.dict(grainsmod.__grains__, {"a": {"b": "bval"}}):
            res = grainsmod.append("a:a_list", "d")
            self.assertEqual(res, {"a": {"a_list": ["d"], "b": "bval"}})
            self.assertEqual(
                grainsmod.__grains__, {"a": {"a_list": ["d"], "b": "bval"}}
            )

        # Append to an existing string, with convert
        with patch.dict(grainsmod.__grains__, {"a": {"b": "bval"}}):
            res = grainsmod.append("a:b", "d", convert=True)
            self.assertEqual(res, {"a": {"b": ["bval", "d"]}})
            self.assertEqual(grainsmod.__grains__, {"a": {"b": ["bval", "d"]}})

        # Append to an existing dict, with convert
        with patch.dict(grainsmod.__grains__, {"a": {"b": {"b1": "bval1"}}}):
            res = grainsmod.append("a:b", "d", convert=True)
            self.assertEqual(res, {"a": {"b": [{"b1": "bval1"}, "d"]}})
            self.assertEqual(grainsmod.__grains__, {"a": {"b": [{"b1": "bval1"}, "d"]}})

    def test_append_to_an_element_of_a_list(self):
        # Append to an element in a list
        # It currently fails silently
        with patch.dict(grainsmod.__grains__, {"a": ["b", "c"]}):
            res = grainsmod.append("a:b", "d")
            self.assertEqual(res, {"a": ["b", "c"]})
            self.assertEqual(grainsmod.__grains__, {"a": ["b", "c"]})

    def test_set_value_already_set(self):
        # Set a grain to the same simple value
        with patch.dict(grainsmod.__grains__, {"a": 12, "c": 8}):
            res = grainsmod.set("a", 12)
            self.assertTrue(res["result"])
            self.assertEqual(res["comment"], "Grain is already set")
            self.assertEqual(grainsmod.__grains__, {"a": 12, "c": 8})

        # Set a grain to the same complex value
        with patch.dict(grainsmod.__grains__, {"a": ["item", 12], "c": 8}):
            res = grainsmod.set("a", ["item", 12])
            self.assertTrue(res["result"])
            self.assertEqual(res["comment"], "Grain is already set")
            self.assertEqual(grainsmod.__grains__, {"a": ["item", 12], "c": 8})

        # Set a key to the same simple value in a nested grain
        with patch.dict(
            grainsmod.__grains__, {"a": "aval", "b": {"nested": "val"}, "c": 8}
        ):
            res = grainsmod.set("b,nested", "val", delimiter=",")
            self.assertTrue(res["result"])
            self.assertEqual(res["comment"], "Grain is already set")
            self.assertEqual(
                grainsmod.__grains__, {"a": "aval", "b": {"nested": "val"}, "c": 8}
            )

    def test_set_fail_replacing_existing_complex_key(self):
        # Fails to set a complex value without 'force'
        with patch.dict(grainsmod.__grains__, {"a": "aval", "c": 8}):
            res = grainsmod.set("a", ["item", 12])
            self.assertFalse(res["result"])
            self.assertEqual(
                res["comment"],
                "The key 'a' exists and the given value is a "
                "dict or a list. Use 'force=True' to overwrite.",
            )
            self.assertEqual(grainsmod.__grains__, {"a": "aval", "c": 8})

        # Fails to overwrite a complex value without 'force'
        with patch.dict(grainsmod.__grains__, {"a": ["item", 12], "c": 8}):
            res = grainsmod.set("a", ["item", 14])
            self.assertFalse(res["result"])
            self.assertEqual(
                res["comment"],
                "The key 'a' exists but is a dict or a list. "
                + "Use 'force=True' to overwrite.",
            )
            self.assertEqual(grainsmod.__grains__, {"a": ["item", 12], "c": 8})

        # Fails to overwrite a complex value without 'force' in a nested grain
        with patch.dict(
            grainsmod.__grains__, {"a": "aval", "b": ["l1", {"l2": ["val1"]}], "c": 8}
        ):
            res = grainsmod.set("b,l2", "val2", delimiter=",")
            self.assertFalse(res["result"])
            self.assertEqual(
                res["comment"],
                "The key 'b,l2' exists but is a dict or a "
                + "list. Use 'force=True' to overwrite.",
            )
            self.assertEqual(
                grainsmod.__grains__,
                {"a": "aval", "b": ["l1", {"l2": ["val1"]}], "c": 8},
            )

    def test_set_nested_fails_replace_simple_value(self):
        # Fails to replace a simple value with a new dictionary consisting
        # of the specified key and value
        with patch.dict(grainsmod.__grains__, {"a": "aval", "b": "l1", "c": 8}):
            res = grainsmod.set("b,l3", "val3", delimiter=",")
            self.assertFalse(res["result"])
            self.assertEqual(
                res["comment"],
                "The key 'b' value is 'l1', which is "
                + "different from the provided key 'l3'. "
                + "Use 'force=True' to overwrite.",
            )
            self.assertEqual(grainsmod.__grains__, {"a": "aval", "b": "l1", "c": 8})

    def test_set_simple_value(self):
        with patch.dict(grainsmod.__grains__, {"a": ["b", "c"], "c": 8}):
            res = grainsmod.set("b", "bval")
            self.assertTrue(res["result"])
            self.assertEqual(res["changes"], {"b": "bval"})
            self.assertEqual(
                grainsmod.__grains__, {"a": ["b", "c"], "b": "bval", "c": 8}
            )

    def test_set_replace_value(self):
        with patch.dict(grainsmod.__grains__, {"a": "aval", "c": 8}):
            res = grainsmod.set("a", 12)
            self.assertTrue(res["result"])
            self.assertEqual(res["changes"], {"a": 12})
            self.assertEqual(grainsmod.__grains__, {"a": 12, "c": 8})

    def test_set_None_ok(self):
        with patch.dict(grainsmod.__grains__, {"a": "aval", "c": 8}):
            res = grainsmod.set("b", None)
            self.assertTrue(res["result"])
            self.assertEqual(res["changes"], {"b": None})
            self.assertEqual(grainsmod.__grains__, {"a": "aval", "b": None, "c": 8})

    def test_set_None_ok_destructive(self):
        with patch.dict(grainsmod.__grains__, {"a": "aval", "c": 8}):
            res = grainsmod.set("b", None, destructive=True)
            self.assertTrue(res["result"])
            self.assertEqual(res["changes"], {"b": None})
            self.assertEqual(grainsmod.__grains__, {"a": "aval", "c": 8})

    def test_set_None_replace_ok(self):
        with patch.dict(grainsmod.__grains__, {"a": "aval", "c": 8}):
            res = grainsmod.set("a", None)
            self.assertTrue(res["result"])
            self.assertEqual(res["changes"], {"a": None})
            self.assertEqual(grainsmod.__grains__, {"a": None, "c": 8})

    def test_set_None_force_destructive(self):
        with patch.dict(grainsmod.__grains__, {"a": "aval", "c": 8}):
            res = grainsmod.set("a", None, force=True, destructive=True)
            self.assertTrue(res["result"])
            self.assertEqual(res["changes"], {"a": None})
            self.assertEqual(grainsmod.__grains__, {"c": 8})

    def test_set_replace_value_was_complex_force(self):
        with patch.dict(grainsmod.__grains__, {"a": ["item", 12], "c": 8}):
            res = grainsmod.set("a", "aval", force=True)
            self.assertTrue(res["result"])
            self.assertEqual(res["changes"], {"a": "aval"})
            self.assertEqual(grainsmod.__grains__, {"a": "aval", "c": 8})

    def test_set_complex_value_force(self):
        with patch.dict(grainsmod.__grains__, {"a": "aval", "c": 8}):
            res = grainsmod.set("a", ["item", 12], force=True)
            self.assertTrue(res["result"])
            self.assertEqual(res["changes"], {"a": ["item", 12]})
            self.assertEqual(grainsmod.__grains__, {"a": ["item", 12], "c": 8})

    def test_set_nested_create(self):
        with patch.dict(grainsmod.__grains__, {"a": "aval", "c": 8}):
            res = grainsmod.set("b,nested", "val", delimiter=",")
            self.assertTrue(res["result"])
            self.assertEqual(res["changes"], {"b": {"nested": "val"}})
            self.assertEqual(
                grainsmod.__grains__, {"a": "aval", "b": {"nested": "val"}, "c": 8}
            )

    def test_set_nested_update_dict(self):
        with patch.dict(
            grainsmod.__grains__, {"a": "aval", "b": {"nested": "val"}, "c": 8}
        ):
            res = grainsmod.set("b,nested", "val2", delimiter=",")
            self.assertTrue(res["result"])
            self.assertEqual(res["changes"], {"b": {"nested": "val2"}})
            self.assertEqual(
                grainsmod.__grains__, {"a": "aval", "b": {"nested": "val2"}, "c": 8}
            )

    def test_set_nested_update_dict_remove_key(self):
        with patch.dict(
            grainsmod.__grains__, {"a": "aval", "b": {"nested": "val"}, "c": 8}
        ):
            res = grainsmod.set("b,nested", None, delimiter=",", destructive=True)
            self.assertTrue(res["result"])
            self.assertEqual(res["changes"], {"b": {}})
            self.assertEqual(grainsmod.__grains__, {"a": "aval", "b": {}, "c": 8})

    def test_set_nested_update_dict_new_key(self):
        with patch.dict(
            grainsmod.__grains__, {"a": "aval", "b": {"nested": "val"}, "c": 8}
        ):
            res = grainsmod.set("b,b2", "val2", delimiter=",")
            self.assertTrue(res["result"])
            self.assertEqual(res["changes"], {"b": {"b2": "val2", "nested": "val"}})
            self.assertEqual(
                grainsmod.__grains__,
                {"a": "aval", "b": {"b2": "val2", "nested": "val"}, "c": 8},
            )

    def test_set_nested_list_replace_key(self):
        with patch.dict(
            grainsmod.__grains__, {"a": "aval", "b": ["l1", "l2", "l3"], "c": 8}
        ):
            res = grainsmod.set("b,l2", "val2", delimiter=",")
            self.assertTrue(res["result"])
            self.assertEqual(res["changes"], {"b": ["l1", {"l2": "val2"}, "l3"]})
            self.assertEqual(
                grainsmod.__grains__,
                {"a": "aval", "b": ["l1", {"l2": "val2"}, "l3"], "c": 8},
            )

    def test_set_nested_list_update_dict_key(self):
        with patch.dict(
            grainsmod.__grains__, {"a": "aval", "b": ["l1", {"l2": "val1"}], "c": 8}
        ):
            res = grainsmod.set("b,l2", "val2", delimiter=",")
            self.assertTrue(res["result"])
            self.assertEqual(res["changes"], {"b": ["l1", {"l2": "val2"}]})
            self.assertEqual(
                grainsmod.__grains__, {"a": "aval", "b": ["l1", {"l2": "val2"}], "c": 8}
            )

    def test_set_nested_list_update_dict_key_overwrite(self):
        with patch.dict(
            grainsmod.__grains__, {"a": "aval", "b": ["l1", {"l2": ["val1"]}], "c": 8}
        ):
            res = grainsmod.set("b,l2", "val2", delimiter=",", force=True)
            self.assertTrue(res["result"])
            self.assertEqual(res["changes"], {"b": ["l1", {"l2": "val2"}]})
            self.assertEqual(
                grainsmod.__grains__, {"a": "aval", "b": ["l1", {"l2": "val2"}], "c": 8}
            )

    def test_set_nested_list_append_dict_key(self):
        with patch.dict(
            grainsmod.__grains__, {"a": "aval", "b": ["l1", {"l2": "val2"}], "c": 8}
        ):
            res = grainsmod.set("b,l3", "val3", delimiter=",")
            self.assertTrue(res["result"])
            self.assertEqual(
                res["changes"], {"b": ["l1", {"l2": "val2"}, {"l3": "val3"}]}
            )
            self.assertEqual(
                grainsmod.__grains__,
                {"a": "aval", "b": ["l1", {"l2": "val2"}, {"l3": "val3"}], "c": 8},
            )

    def test_set_nested_existing_value_is_the_key(self):
        with patch.dict(grainsmod.__grains__, {"a": "aval", "b": "l3", "c": 8}):
            res = grainsmod.set("b,l3", "val3", delimiter=",")
            self.assertTrue(res["result"])
            self.assertEqual(res["changes"], {"b": {"l3": "val3"}})
            self.assertEqual(
                grainsmod.__grains__, {"a": "aval", "b": {"l3": "val3"}, "c": 8}
            )

    def test_set_nested_existing_value_overwrite(self):
        with patch.dict(grainsmod.__grains__, {"a": "aval", "b": "l1", "c": 8}):
            res = grainsmod.set("b,l3", "val3", delimiter=",", force=True)
            self.assertTrue(res["result"])
            self.assertEqual(res["changes"], {"b": {"l3": "val3"}})
            self.assertEqual(
                grainsmod.__grains__, {"a": "aval", "b": {"l3": "val3"}, "c": 8}
            )

    def test_set_deeply_nested_update(self):
        with patch.dict(
            grainsmod.__grains__,
            {"a": "aval", "b": {"l1": ["l21", "l22", {"l23": "l23val"}]}, "c": 8},
        ):
            res = grainsmod.set("b,l1,l23", "val", delimiter=",")
            self.assertTrue(res["result"])
            self.assertEqual(
                res["changes"], {"b": {"l1": ["l21", "l22", {"l23": "val"}]}}
            )
            self.assertEqual(
                grainsmod.__grains__,
                {"a": "aval", "b": {"l1": ["l21", "l22", {"l23": "val"}]}, "c": 8},
            )

    def test_set_deeply_nested_create(self):
        with patch.dict(
            grainsmod.__grains__,
            {"a": "aval", "b": {"l1": ["l21", "l22", {"l23": "l23val"}]}, "c": 8},
        ):
            res = grainsmod.set("b,l1,l24,l241", "val", delimiter=",")
            self.assertTrue(res["result"])
            self.assertEqual(
                res["changes"],
                {
                    "b": {
                        "l1": [
                            "l21",
                            "l22",
                            {"l23": "l23val"},
                            {"l24": {"l241": "val"}},
                        ]
                    }
                },
            )
            self.assertEqual(
                grainsmod.__grains__,
                {
                    "a": "aval",
                    "b": {
                        "l1": [
                            "l21",
                            "l22",
                            {"l23": "l23val"},
                            {"l24": {"l241": "val"}},
                        ]
                    },
                    "c": 8,
                },
            )

    def test_get_ordered(self):
        with patch.dict(
            grainsmod.__grains__,
            OrderedDict(
                [
                    ("a", "aval"),
                    (
                        "b",
                        OrderedDict(
                            [
                                ("z", "zval"),
                                (
                                    "l1",
                                    ["l21", "l22", OrderedDict([("l23", "l23val")])],
                                ),
                            ]
                        ),
                    ),
                    ("c", 8),
                ]
            ),
        ):
            res = grainsmod.get("b")
            self.assertEqual(type(res), OrderedDict)
            # Check that order really matters
            self.assertTrue(
                res
                == OrderedDict(
                    [
                        ("z", "zval"),
                        ("l1", ["l21", "l22", OrderedDict([("l23", "l23val")])]),
                    ]
                )
            )
            self.assertFalse(
                res
                == OrderedDict(
                    [
                        ("l1", ["l21", "l22", OrderedDict([("l23", "l23val")])]),
                        ("z", "zval"),
                    ]
                )
            )

    def test_get_unordered(self):
        with patch.dict(
            grainsmod.__grains__,
            OrderedDict(
                [
                    ("a", "aval"),
                    (
                        "b",
                        OrderedDict(
                            [
                                ("z", "zval"),
                                (
                                    "l1",
                                    ["l21", "l22", OrderedDict([("l23", "l23val")])],
                                ),
                            ]
                        ),
                    ),
                    ("c", 8),
                ]
            ),
        ):
            res = grainsmod.get("b", ordered=False)
            self.assertEqual(type(res), dict)
            # Check that order doesn't matter
            self.assertTrue(
                res
                == OrderedDict(
                    [
                        ("l1", ["l21", "l22", OrderedDict([("l23", "l23val")])]),
                        ("z", "zval"),
                    ]
                )
            )

    def test_equals(self):
        with patch.dict(
            grainsmod.__grains__,
            OrderedDict(
                [
                    ("a", "aval"),
                    (
                        "b",
                        OrderedDict(
                            [
                                ("z", "zval"),
                                (
                                    "l1",
                                    ["l21", "l22", OrderedDict([("l23", "l23val")])],
                                ),
                            ]
                        ),
                    ),
                    ("c", 8),
                ]
            ),
        ):
            res = grainsmod.equals("a", "aval")
            self.assertEqual(type(res), bool)
            self.assertTrue(res)
            res = grainsmod.equals("b:z", "zval")
            self.assertTrue(res)
            res = grainsmod.equals("b:z", "aval")
            self.assertFalse(res)

    def test_grains_setval_refresh_pillar(self):
        """
        Test that refresh_pillar kwarg is being passed correctly from grains.setval to saltutil.refresh_grains
        """
        ret = grainsmod.setval("test_grains_setval_refresh_pillar", "saltopotamus")
        grainsmod.__salt__["saltutil.refresh_grains"].assert_called_with(
            refresh_pillar=True
        )
        ret = grainsmod.setval(
            "test_grains_setval_refresh_pillar", "saltopotamus", refresh_pillar=True
        )
        grainsmod.__salt__["saltutil.refresh_grains"].assert_called_with(
            refresh_pillar=True
        )
        ret = grainsmod.setval(
            "test_grains_setval_refresh_pillar", "saltopotamus", refresh_pillar=False
        )
        grainsmod.__salt__["saltutil.refresh_grains"].assert_called_with(
            refresh_pillar=False
        )

    def test_setval_unicode(self):
        key = "塩"  # salt
        value = "塩人生です"  # salt is life

        # Note: call setvals 2 times is important
        # 1: add key to conf grains
        # 2: update and read key from conf grains
        for _ in range(2):
            ret = grainsmod.setvals({key: value})
            self.assertIn(key, ret)
            self.assertEqual(ret[key], value)

    def test_delval_single(self):
        with patch.dict(
            grainsmod.__grains__, {"a": "aval", "b": {"nested": "val"}, "c": 8}
        ):
            res = grainsmod.delval("a")
            self.assertTrue(res["result"])
            self.assertEqual(res["changes"], {"a": None})
            self.assertEqual(
                grainsmod.__grains__, {"a": None, "b": {"nested": "val"}, "c": 8}
            )

    def test_delval_nested(self):
        with patch.dict(
            grainsmod.__grains__, {"a": "aval", "b": {"nested": "val"}, "c": 8}
        ):
            res = grainsmod.delval("b:nested")
            self.assertTrue(res["result"])
            self.assertEqual(res["changes"], {"b": {"nested": None}})
            self.assertEqual(
                grainsmod.__grains__, {"a": "aval", "b": {"nested": None}, "c": 8}
            )

    def test_delkey_single_key(self):
        with patch.dict(
            grainsmod.__grains__, {"a": "aval", "b": {"nested": "val"}, "c": 8}
        ):
            res = grainsmod.delkey("a")
            self.assertTrue(res["result"])
            self.assertEqual(res["changes"], {"a": None})
            self.assertEqual(grainsmod.__grains__, {"b": {"nested": "val"}, "c": 8})

    def test_delkey_nested_key(self):
        with patch.dict(
            grainsmod.__grains__, {"a": "aval", "b": {"nested": "val"}, "c": 8}
        ):
            res = grainsmod.delkey("b:nested")
            self.assertTrue(res["result"])
            self.assertEqual(res["changes"], {"b": {}})
            self.assertEqual(grainsmod.__grains__, {"a": "aval", "b": {}, "c": 8})



# -*- coding: utf-8 -*-
"""
    tests.integration.returners.test_noop_return
    ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

    This test module is meant to cover the issue being fixed by:

        https://github.com/saltstack/salt/pull/54731
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import logging

# Import 3rd-party tests
import salt.ext.six as six

# Import Salt Testing libs
from tests.support.case import ModuleCase
from tests.support.helpers import TstSuiteLoggingHandler
from tests.support.unit import skipIf

log = logging.getLogger(__name__)


@skipIf(six.PY3, "Runtest Log Hander Disabled for PY3, #41836")
class TestEventReturn(ModuleCase):
    def test_noop_return(self):
        with TstSuiteLoggingHandler(
            format="%(message)s", level=logging.DEBUG
        ) as handler:
            self.run_function("test.ping")
            assert (
                any("NOOP_RETURN" in s for s in handler.messages) is True
            ), "NOOP_RETURN not found in log messages"



# -*- coding: utf-8 -*-
"""
Module for handling openstack glance calls.

:optdepends:    - glanceclient Python adapter
:configuration: This module is not usable until the following are specified
    either in a pillar or in the minion's config file::

        keystone.user: admin
        keystone.password: verybadpass
        keystone.tenant: admin
        keystone.insecure: False   #(optional)
        keystone.auth_url: 'http://127.0.0.1:5000/v2.0/'

    If configuration for multiple openstack accounts is required, they can be
    set up as different configuration profiles:
    For example::

        openstack1:
          keystone.user: admin
          keystone.password: verybadpass
          keystone.tenant: admin
          keystone.auth_url: 'http://127.0.0.1:5000/v2.0/'

        openstack2:
          keystone.user: admin
          keystone.password: verybadpass
          keystone.tenant: admin
          keystone.auth_url: 'http://127.0.0.2:5000/v2.0/'

    With this configuration in place, any of the glance functions can
    make use of a configuration profile by declaring it explicitly.
    For example::

        salt '*' glance.image_list profile=openstack1
"""

# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

import logging
import pprint
import re

# Import salt libs
from salt.exceptions import SaltInvocationError
from salt.ext import six

# pylint: disable=import-error
HAS_GLANCE = False
try:
    from glanceclient import client
    from glanceclient import exc

    HAS_GLANCE = True
except ImportError:
    pass

# Workaround, as the Glance API v2 requires you to
# already have a keystone session token
HAS_KEYSTONE = False
try:
    from keystoneclient.v2_0 import client as kstone

    # import keystoneclient.apiclient.exceptions as kstone_exc
    HAS_KEYSTONE = True
except ImportError:
    pass


logging.basicConfig(level=logging.DEBUG)
log = logging.getLogger(__name__)


def __virtual__():
    """
    Only load this module if glance
    is installed on this minion.
    """
    if HAS_GLANCE:
        return "glance"
    return (
        False,
        "The glance execution module cannot be loaded: the glanceclient python library is not available.",
    )


__opts__ = {}


def _auth(profile=None, api_version=2, **connection_args):
    """
    Set up glance credentials, returns
    `glanceclient.client.Client`. Optional parameter
    "api_version" defaults to 2.

    Only intended to be used within glance-enabled modules
    """
    __utils__["versions.warn_until"](
        "Aluminium",
        (
            "The glance module has been deprecated and will be removed in {version}.  "
            "Please update to using the glanceng module"
        ),
    )

    if profile:
        prefix = profile + ":keystone."
    else:
        prefix = "keystone."

    def get(key, default=None):
        """
        Checks connection_args, then salt-minion config,
        falls back to specified default value.
        """
        return connection_args.get(
            "connection_" + key, __salt__["config.get"](prefix + key, default)
        )

    user = get("user", "admin")
    password = get("password", None)
    tenant = get("tenant", "admin")
    tenant_id = get("tenant_id")
    auth_url = get("auth_url", "http://127.0.0.1:35357/v2.0")
    insecure = get("insecure", False)
    admin_token = get("token")
    region = get("region")
    ks_endpoint = get("endpoint", "http://127.0.0.1:9292/")
    g_endpoint_url = __salt__["keystone.endpoint_get"]("glance", profile)
    # The trailing 'v2' causes URLs like thise one:
    # http://127.0.0.1:9292/v2/v1/images
    g_endpoint_url = re.sub("/v2", "", g_endpoint_url["internalurl"])

    if admin_token and api_version != 1 and not password:
        # If we had a password we could just
        # ignore the admin-token and move on...
        raise SaltInvocationError(
            "Only can use keystone admin token " + "with Glance API v1"
        )
    elif password:
        # Can't use the admin-token anyway
        kwargs = {
            "username": user,
            "password": password,
            "tenant_id": tenant_id,
            "auth_url": auth_url,
            "endpoint_url": g_endpoint_url,
            "region_name": region,
            "tenant_name": tenant,
        }
        # 'insecure' keyword not supported by all v2.0 keystone clients
        #   this ensures it's only passed in when defined
        if insecure:
            kwargs["insecure"] = True
    elif api_version == 1 and admin_token:
        kwargs = {
            "token": admin_token,
            "auth_url": auth_url,
            "endpoint_url": g_endpoint_url,
        }
    else:
        raise SaltInvocationError("No credentials to authenticate with.")

    if HAS_KEYSTONE:
        log.debug(
            "Calling keystoneclient.v2_0.client.Client(%s, **%s)", ks_endpoint, kwargs
        )
        keystone = kstone.Client(**kwargs)
        kwargs["token"] = keystone.get_token(keystone.session)
        # This doesn't realy prevent the password to show up
        # in the minion log as keystoneclient.session is
        # logging it anyway when in debug-mode
        kwargs.pop("password")
        log.debug(
            "Calling glanceclient.client.Client(%s, %s, **%s)",
            api_version,
            g_endpoint_url,
            kwargs,
        )
        # may raise exc.HTTPUnauthorized, exc.HTTPNotFound
        # but we deal with those elsewhere
        return client.Client(api_version, g_endpoint_url, **kwargs)
    else:
        raise NotImplementedError("Can't retrieve a auth_token without keystone")


def _add_image(collection, image):
    """
    Add image to given dictionary
    """
    image_prep = {
        "id": image.id,
        "name": image.name,
        "created_at": image.created_at,
        "file": image.file,
        "min_disk": image.min_disk,
        "min_ram": image.min_ram,
        "owner": image.owner,
        "protected": image.protected,
        "status": image.status,
        "tags": image.tags,
        "updated_at": image.updated_at,
        "visibility": image.visibility,
    }
    # Those cause AttributeErrors in Icehouse' glanceclient
    for attr in ["container_format", "disk_format", "size"]:
        if attr in image:
            image_prep[attr] = image[attr]
    if type(collection) is dict:
        collection[image.name] = image_prep
    elif type(collection) is list:
        collection.append(image_prep)
    else:
        msg = (
            '"collection" is {0}'.format(type(collection)) + "instead of dict or list."
        )
        log.error(msg)
        raise TypeError(msg)
    return collection


def image_create(
    name,
    location=None,
    profile=None,
    visibility=None,
    container_format="bare",
    disk_format="raw",
    protected=None,
):
    """
    Create an image (glance image-create)

    CLI Example, old format:

    .. code-block:: bash

        salt '*' glance.image_create name=f16-jeos \\
                 disk_format=qcow2 container_format=ovf

    CLI Example, new format resembling Glance API v2:

    .. code-block:: bash

        salt '*' glance.image_create name=f16-jeos visibility=public \\
                 disk_format=qcow2 container_format=ovf

    The parameter 'visibility' defaults to 'public' if not specified.
    """
    kwargs = {}
    # valid options for "visibility":
    v_list = ["public", "private"]
    # valid options for "container_format":
    cf_list = ["ami", "ari", "aki", "bare", "ovf"]
    # valid options for "disk_format":
    df_list = ["ami", "ari", "aki", "vhd", "vmdk", "raw", "qcow2", "vdi", "iso"]

    kwargs["copy_from"] = location
    if visibility is not None:
        if visibility not in v_list:
            raise SaltInvocationError(
                '"visibility" needs to be one '
                + "of the following: {0}".format(", ".join(v_list))
            )
        elif visibility == "public":
            kwargs["is_public"] = True
        else:
            kwargs["is_public"] = False
    else:
        kwargs["is_public"] = True
    if container_format not in cf_list:
        raise SaltInvocationError(
            '"container_format" needs to be '
            + "one of the following: {0}".format(", ".join(cf_list))
        )
    else:
        kwargs["container_format"] = container_format
    if disk_format not in df_list:
        raise SaltInvocationError(
            '"disk_format" needs to be one '
            + "of the following: {0}".format(", ".join(df_list))
        )
    else:
        kwargs["disk_format"] = disk_format
    if protected is not None:
        kwargs["protected"] = protected
    # Icehouse's glanceclient doesn't have add_location() and
    # glanceclient.v2 doesn't implement Client.images.create()
    # in a usable fashion. Thus we have to use v1 for now.
    g_client = _auth(profile, api_version=1)
    image = g_client.images.create(name=name, **kwargs)
    return image_show(image.id, profile=profile)


def image_delete(id=None, name=None, profile=None):  # pylint: disable=C0103
    """
    Delete an image (glance image-delete)

    CLI Examples:

    .. code-block:: bash

        salt '*' glance.image_delete c2eb2eb0-53e1-4a80-b990-8ec887eae7df
        salt '*' glance.image_delete id=c2eb2eb0-53e1-4a80-b990-8ec887eae7df
        salt '*' glance.image_delete name=f16-jeos
    """
    g_client = _auth(profile)
    image = {"id": False, "name": None}
    if name:
        for image in g_client.images.list():
            if image.name == name:
                id = image.id  # pylint: disable=C0103
                continue
    if not id:
        return {
            "result": False,
            "comment": "Unable to resolve image id " "for name {0}".format(name),
        }
    elif not name:
        name = image["name"]
    try:
        g_client.images.delete(id)
    except exc.HTTPNotFound:
        return {"result": False, "comment": "No image with ID {0}".format(id)}
    except exc.HTTPForbidden as forbidden:
        log.error(six.text_type(forbidden))
        return {"result": False, "comment": six.text_type(forbidden)}
    return {
        "result": True,
        "comment": "Deleted image '{0}' ({1}).".format(name, id),
    }


def image_show(id=None, name=None, profile=None):  # pylint: disable=C0103
    """
    Return details about a specific image (glance image-show)

    CLI Example:

    .. code-block:: bash

        salt '*' glance.image_show
    """
    g_client = _auth(profile)
    ret = {}
    if name:
        for image in g_client.images.list():
            if image.name == name:
                id = image.id  # pylint: disable=C0103
                continue
    if not id:
        return {
            "result": False,
            "comment": "Unable to resolve image ID " "for name '{0}'".format(name),
        }
    try:
        image = g_client.images.get(id)
    except exc.HTTPNotFound:
        return {"result": False, "comment": "No image with ID {0}".format(id)}
    pformat = pprint.PrettyPrinter(indent=4).pformat
    log.debug(
        "Properties of image %s:\n%s",
        image.name,
        pprint.PrettyPrinter(indent=4).pformat(image),
    )

    schema = image_schema(profile=profile)
    if len(schema.keys()) == 1:
        schema = schema["image"]
    for key in schema:
        if key in image:
            ret[key] = image[key]
    return ret


def image_list(id=None, profile=None, name=None):  # pylint: disable=C0103
    """
    Return a list of available images (glance image-list)

    CLI Example:

    .. code-block:: bash

        salt '*' glance.image_list
    """
    g_client = _auth(profile)
    ret = []
    for image in g_client.images.list():
        if id is None and name is None:
            _add_image(ret, image)
        else:
            if id is not None and id == image.id:
                _add_image(ret, image)
                return ret
            if name == image.name:
                if name in ret and __salt__["salt_version.less_than"]("Boron"):
                    # Not really worth an exception
                    return {
                        "result": False,
                        "comment": "More than one image with "
                        'name "{0}"'.format(name),
                    }
                _add_image(ret, image)
    log.debug("Returning images: %s", ret)
    return ret


def image_schema(profile=None):
    """
    Returns names and descriptions of the schema "image"'s
    properties for this profile's instance of glance

    CLI Example:

    .. code-block:: bash

        salt '*' glance.image_schema
    """
    return schema_get("image", profile)


def image_update(id=None, name=None, profile=None, **kwargs):  # pylint: disable=C0103
    """
    Update properties of given image.
    Known to work for:
    - min_ram (in MB)
    - protected (bool)
    - visibility ('public' or 'private')

    CLI Example:

    .. code-block:: bash

        salt '*' glance.image_update id=c2eb2eb0-53e1-4a80-b990-8ec887eae7df
        salt '*' glance.image_update name=f16-jeos
    """
    if id:
        image = image_show(id=id, profile=profile)
        if "result" in image and not image["result"]:
            return image
        elif len(image) == 1:
            image = next(iter(image.values()))
    elif name:
        img_list = image_list(name=name, profile=profile)
        if img_list is dict and "result" in img_list:
            return img_list
        elif not img_list:
            return {
                "result": False,
                "comment": "No image with name '{0}' " "found.".format(name),
            }
        elif len(img_list) == 1:
            try:
                image = img_list[0]
            except KeyError:
                image = img_list[name]
    else:
        raise SaltInvocationError
    log.debug("Found image:\n%s", image)
    to_update = {}
    for key, value in kwargs.items():
        if key.startswith("_"):
            continue
        if key not in image or image[key] != value:
            log.debug("add <%s=%s> to to_update", key, value)
            to_update[key] = value
    g_client = _auth(profile)
    updated = g_client.images.update(image["id"], **to_update)
    return updated


def schema_get(name, profile=None):
    """
    Known valid names of schemas are:
      - image
      - images
      - member
      - members

    CLI Example:

    .. code-block:: bash

        salt '*' glance.schema_get name=f16-jeos
    """
    g_client = _auth(profile)
    schema_props = {}
    for prop in g_client.schemas.get(name).properties:
        schema_props[prop.name] = prop.description
    log.debug(
        "Properties of schema %s:\n%s",
        name,
        pprint.PrettyPrinter(indent=4).pformat(schema_props),
    )
    return {name: schema_props}


def _item_list(profile=None):
    """
    Template for writing list functions
    Return a list of available items (glance items-list)

    CLI Example:

    .. code-block:: bash

        salt '*' glance.item_list
    """
    g_client = _auth(profile)
    ret = []
    for item in g_client.items.list():
        ret.append(item.__dict__)
        # ret[item.name] = {
        #        'name': item.name,
        #    }
    return ret


# The following is a list of functions that need to be incorporated in the
# glance module. This list should be updated as functions are added.

# image-download      Download a specific image.
# member-create       Share a specific image with a tenant.
# member-delete       Remove a shared image from a tenant.
# member-list         Describe sharing permissions by image or tenant.



# -*- coding: utf-8 -*-
"""
    :codeauthor: Andrew Colin Kissa <andrew@topdog.za.net>
"""
from __future__ import absolute_import, print_function, unicode_literals

import salt.states.postgres_language as postgres_language
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class PostgresLanguageTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.postgres_language
    """

    def setup_loader_modules(self):
        return {postgres_language: {}}

    def setUp(self):
        """
        Setup data for the tests
        """
        self.name = "plpgsql"
        self.ret = {"name": self.name, "changes": {}, "result": False, "comment": ""}
        self.mock_true = MagicMock(return_value=True)
        self.mock_false = MagicMock(return_value=False)
        self.mock_empty_language_list = MagicMock(return_value={})
        self.mock_language_list = MagicMock(return_value={"plpgsql": self.name})

    def tearDown(self):
        del self.ret
        del self.mock_true
        del self.mock_false
        del self.mock_language_list
        del self.mock_empty_language_list

    def test_present_existing(self):
        """
        Test present, language is already present in database
        """
        with patch.dict(
            postgres_language.__salt__,
            {"postgres.language_list": self.mock_language_list},
        ):
            comt = "Language {0} is already installed".format(self.name)
            self.ret.update({"comment": comt, "result": True})
            self.assertDictEqual(
                postgres_language.present(self.name, "testdb"), self.ret
            )

    def test_present_non_existing_pass(self):
        """
        Test present, language not present in database - pass
        """
        with patch.dict(
            postgres_language.__salt__,
            {
                "postgres.language_list": self.mock_empty_language_list,
                "postgres.language_create": self.mock_true,
            },
        ):
            with patch.dict(postgres_language.__opts__, {"test": True}):
                comt = "Language {0} is set to be installed".format(self.name)
                self.ret.update({"comment": comt, "result": None})
                self.assertDictEqual(
                    postgres_language.present(self.name, "testdb"), self.ret
                )

            with patch.dict(postgres_language.__opts__, {"test": False}):
                comt = "Language {0} has been installed".format(self.name)
                self.ret.update(
                    {"comment": comt, "result": True, "changes": {"plpgsql": "Present"}}
                )
                self.assertDictEqual(
                    postgres_language.present(self.name, "testdb"), self.ret
                )

    def test_present_non_existing_fail(self):
        """
        Test present, language not present in database - fail
        """
        with patch.dict(
            postgres_language.__salt__,
            {
                "postgres.language_list": self.mock_empty_language_list,
                "postgres.language_create": self.mock_false,
            },
        ):
            with patch.dict(postgres_language.__opts__, {"test": True}):
                comt = "Language {0} is set to be installed".format(self.name)
                self.ret.update({"comment": comt, "result": None})
                self.assertDictEqual(
                    postgres_language.present(self.name, "testdb"), self.ret
                )

            with patch.dict(postgres_language.__opts__, {"test": False}):
                comt = "Failed to install language {0}".format(self.name)
                self.ret.update({"comment": comt, "result": False})
                self.assertDictEqual(
                    postgres_language.present(self.name, "testdb"), self.ret
                )

    def test_absent_existing(self):
        """
        Test absent, language present in database
        """
        with patch.dict(
            postgres_language.__salt__,
            {
                "postgres.language_exists": self.mock_true,
                "postgres.language_remove": self.mock_true,
            },
        ):
            with patch.dict(postgres_language.__opts__, {"test": True}):
                comt = "Language {0} is set to be removed".format(self.name)
                self.ret.update({"comment": comt, "result": None})
                self.assertDictEqual(
                    postgres_language.absent(self.name, "testdb"), self.ret
                )

            with patch.dict(postgres_language.__opts__, {"test": False}):
                comt = "Language {0} has been removed".format(self.name)
                self.ret.update(
                    {"comment": comt, "result": True, "changes": {"plpgsql": "Absent"}}
                )
                self.assertDictEqual(
                    postgres_language.absent(self.name, "testdb"), self.ret
                )

    def test_absent_non_existing(self):
        """
        Test absent, language not present in database
        """
        with patch.dict(
            postgres_language.__salt__, {"postgres.language_exists": self.mock_false}
        ):
            with patch.dict(postgres_language.__opts__, {"test": True}):
                comt = "Language {0} is not present so " "it cannot be removed".format(
                    self.name
                )
                self.ret.update({"comment": comt, "result": True})
                self.assertDictEqual(
                    postgres_language.absent(self.name, "testdb"), self.ret
                )



# -*- coding: utf-8 -*-
"""Test cases for the ``ldap`` state module

This code is gross.  I started out trying to remove some of the
duplicate code in the test cases, and before I knew it the test code
was an ugly second implementation.

I'm leaving it for now, but this should really be gutted and replaced
with something sensible.
"""

from __future__ import absolute_import, print_function, unicode_literals

import copy

import salt.states.ldap
from salt.ext import six
from salt.utils.oset import OrderedSet
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.unit import TestCase

# emulates the LDAP database.  each key is the DN of an entry and it
# maps to a dict which maps attribute names to sets of values.
db = {}


def _init_db(newdb=None):
    if newdb is None:
        newdb = {}
    global db
    db = newdb


def _complex_db():
    return {
        "dnfoo": {
            "attrfoo1": OrderedSet(("valfoo1.1", "valfoo1.2",)),
            "attrfoo2": OrderedSet(("valfoo2.1",)),
        },
        "dnbar": {
            "attrbar1": OrderedSet(("valbar1.1", "valbar1.2",)),
            "attrbar2": OrderedSet(("valbar2.1",)),
        },
    }


class _dummy_ctx(object):
    def __init__(self):
        pass

    def __enter__(self):
        return self

    def __exit__(self, *exc):
        pass


def _dummy_connect(connect_spec):
    return _dummy_ctx()


def _dummy_search(connect_spec, base, scope):
    if base not in db:
        return {}
    return {
        base: dict(
            ((attr, list(db[base][attr])) for attr in db[base] if len(db[base][attr]))
        )
    }


def _dummy_add(connect_spec, dn, attributes):
    assert dn not in db
    assert attributes
    db[dn] = {}
    for attr, vals in six.iteritems(attributes):
        assert vals
        db[dn][attr] = OrderedSet(vals)
    return True


def _dummy_delete(connect_spec, dn):
    assert dn in db
    del db[dn]
    return True


def _dummy_change(connect_spec, dn, before, after):
    assert before != after
    assert before
    assert after
    assert dn in db
    e = db[dn]
    assert e == before
    all_attrs = OrderedSet()
    all_attrs.update(before)
    all_attrs.update(after)
    directives = []
    for attr in all_attrs:
        if attr not in before:
            assert attr in after
            assert after[attr]
            directives.append(("add", attr, after[attr]))
        elif attr not in after:
            assert attr in before
            assert before[attr]
            directives.append(("delete", attr, ()))
        else:
            assert before[attr]
            assert after[attr]
            to_del = before[attr] - after[attr]
            if to_del:
                directives.append(("delete", attr, to_del))
            to_add = after[attr] - before[attr]
            if to_add:
                directives.append(("add", attr, to_add))
    return _dummy_modify(connect_spec, dn, directives)


def _dummy_modify(connect_spec, dn, directives):
    assert dn in db
    e = db[dn]
    for op, attr, vals in directives:
        if op == "add":
            assert vals
            existing_vals = e.setdefault(attr, OrderedSet())
            for val in vals:
                assert val not in existing_vals
                existing_vals.add(val)
        elif op == "delete":
            assert attr in e
            existing_vals = e[attr]
            assert existing_vals
            if not vals:
                del e[attr]
                continue
            for val in vals:
                assert val in existing_vals
                existing_vals.remove(val)
            if not existing_vals:
                del e[attr]
        elif op == "replace":
            e.pop(attr, None)
            e[attr] = OrderedSet(vals)
        else:
            raise ValueError()
    return True


def _dump_db(d=None):
    if d is None:
        d = db
    return dict(((dn, dict(((attr, list(d[dn][attr])) for attr in d[dn]))) for dn in d))


class LDAPTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        salt_dunder = {}
        for fname in ("connect", "search", "add", "delete", "change", "modify"):
            salt_dunder["ldap3.{}".format(fname)] = globals()["_dummy_" + fname]
        return {
            salt.states.ldap: {"__opts__": {"test": False}, "__salt__": salt_dunder}
        }

    def _test_helper(self, init_db, expected_ret, replace, delete_others=False):
        _init_db(copy.deepcopy(init_db))
        old = _dump_db()
        new = _dump_db()
        expected_db = copy.deepcopy(init_db)
        for dn, attrs in six.iteritems(replace):
            for attr, vals in six.iteritems(attrs):
                if vals:
                    new.setdefault(dn, {})[attr] = list(OrderedSet(vals))
                    expected_db.setdefault(dn, {})[attr] = OrderedSet(vals)
                elif dn in expected_db:
                    new[dn].pop(attr, None)
                    expected_db[dn].pop(attr, None)
            if not expected_db.get(dn, {}):
                new.pop(dn, None)
                expected_db.pop(dn, None)
        if delete_others:
            dn_to_delete = OrderedSet()
            for dn, attrs in six.iteritems(expected_db):
                if dn in replace:
                    to_delete = OrderedSet()
                    for attr, vals in six.iteritems(attrs):
                        if attr not in replace[dn]:
                            to_delete.add(attr)
                    for attr in to_delete:
                        del attrs[attr]
                        del new[dn][attr]
                    if not attrs:
                        dn_to_delete.add(dn)
            for dn in dn_to_delete:
                del new[dn]
                del expected_db[dn]
        name = "ldapi:///"
        expected_ret["name"] = name
        expected_ret.setdefault("result", True)
        expected_ret.setdefault("comment", "Successfully updated LDAP entries")
        expected_ret.setdefault(
            "changes",
            dict(
                (
                    (
                        dn,
                        {
                            "old": dict(
                                (attr, vals)
                                for attr, vals in six.iteritems(old[dn])
                                if vals != new.get(dn, {}).get(attr, ())
                            )
                            if dn in old
                            else None,
                            "new": dict(
                                (attr, vals)
                                for attr, vals in six.iteritems(new[dn])
                                if vals != old.get(dn, {}).get(attr, ())
                            )
                            if dn in new
                            else None,
                        },
                    )
                    for dn in replace
                    if old.get(dn, {}) != new.get(dn, {})
                )
            ),
        )
        entries = [
            {dn: [{"replace": attrs}, {"delete_others": delete_others}]}
            for dn, attrs in six.iteritems(replace)
        ]
        actual = salt.states.ldap.managed(name, entries)
        self.assertDictEqual(expected_ret, actual)
        self.assertDictEqual(expected_db, db)

    def _test_helper_success(self, init_db, replace, delete_others=False):
        self._test_helper(init_db, {}, replace, delete_others)

    def _test_helper_nochange(self, init_db, replace, delete_others=False):
        expected = {
            "changes": {},
            "comment": "LDAP entries already set",
        }
        self._test_helper(init_db, expected, replace, delete_others)

    def test_managed_empty(self):
        _init_db()
        name = "ldapi:///"
        expected = {
            "name": name,
            "changes": {},
            "result": True,
            "comment": "LDAP entries already set",
        }
        actual = salt.states.ldap.managed(name, {})
        self.assertDictEqual(expected, actual)

    def test_managed_add_entry(self):
        self._test_helper_success({}, {"dummydn": {"foo": ["bar", "baz"]}})

    def test_managed_add_attr(self):
        self._test_helper_success(_complex_db(), {"dnfoo": {"attrfoo3": ["valfoo3.1"]}})

    def test_managed_simplereplace(self):
        self._test_helper_success(_complex_db(), {"dnfoo": {"attrfoo1": ["valfoo1.3"]}})

    def test_managed_deleteattr(self):
        self._test_helper_success(_complex_db(), {"dnfoo": {"attrfoo1": []}})

    def test_managed_deletenonexistattr(self):
        self._test_helper_nochange(_complex_db(), {"dnfoo": {"dummyattr": []}})

    def test_managed_deleteentry(self):
        self._test_helper_success(_complex_db(), {"dnfoo": {}}, True)

    def test_managed_deletenonexistentry(self):
        self._test_helper_nochange(_complex_db(), {"dummydn": {}}, True)

    def test_managed_deletenonexistattrinnonexistentry(self):
        self._test_helper_nochange(_complex_db(), {"dummydn": {"dummyattr": []}})

    def test_managed_add_attr_delete_others(self):
        self._test_helper_success(
            _complex_db(), {"dnfoo": {"dummyattr": ["dummyval"]}}, True
        )

    def test_managed_no_net_change(self):
        self._test_helper_nochange(
            _complex_db(), {"dnfoo": {"attrfoo1": ["valfoo1.1", "valfoo1.2"]}}
        )

    def test_managed_repeated_values(self):
        self._test_helper_success(
            {}, {"dummydn": {"dummyattr": ["dummyval", "dummyval"]}}
        )



# -*- coding: utf-8 -*-

# Import python libs
from __future__ import absolute_import, print_function, unicode_literals

import logging

# Import Salt Testing libs
from tests.support.case import ModuleCase, ShellCase
from tests.support.helpers import slowTest

log = logging.getLogger(__name__)


class ExecutorTest(ModuleCase, ShellCase):
    def setup(self):
        self.run_function("saltutil.sync_all")

    @slowTest
    def test_executor(self):
        """
        test that dunders are set
        """
        data = self.run_call("test.arg --module-executors=arg")
        self.assertIn("test.arg fired", "".join(data))



# -*- coding: utf-8 -*-

# import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.boto_secgroup as boto_secgroup
from salt.utils.odict import OrderedDict
from tests.support.case import TestCase

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin


class Boto_SecgroupTestCase(TestCase, LoaderModuleMockMixin):
    """
    TestCase for salt.states.boto_secgroup module
    """

    def setup_loader_modules(self):
        return {boto_secgroup: {}}

    def test__get_rule_changes_no_rules_no_change(self):
        """
        tests a condition with no rules in present or desired group
        """
        present_rules = []
        desired_rules = []
        self.assertEqual(
            boto_secgroup._get_rule_changes(desired_rules, present_rules), ([], [])
        )

    def test__get_rule_changes_create_rules(self):
        """
        tests a condition where a rule must be created
        """
        present_rules = [
            OrderedDict(
                [
                    ("ip_protocol", "tcp"),
                    ("from_port", 22),
                    ("to_port", 22),
                    ("cidr_ip", "0.0.0.0/0"),
                ]
            )
        ]
        desired_rules = [
            OrderedDict(
                [
                    ("ip_protocol", "tcp"),
                    ("from_port", 22),
                    ("to_port", 22),
                    ("cidr_ip", "0.0.0.0/0"),
                ]
            ),
            OrderedDict(
                [
                    ("ip_protocol", "tcp"),
                    ("from_port", 80),
                    ("to_port", 80),
                    ("cidr_ip", "0.0.0.0/0"),
                ]
            ),
        ]
        # can also use: rules_to_create = [rule for rule in desired_rules if rule not in present_rules]
        rules_to_create = [
            OrderedDict(
                [
                    ("ip_protocol", "tcp"),
                    ("from_port", 80),
                    ("to_port", 80),
                    ("cidr_ip", "0.0.0.0/0"),
                ]
            )
        ]
        self.assertEqual(
            boto_secgroup._get_rule_changes(desired_rules, present_rules),
            ([], rules_to_create),
        )

    def test__get_rule_changes_delete_rules(self):
        """
        tests a condition where a rule must be deleted
        """
        present_rules = [
            OrderedDict(
                [
                    ("ip_protocol", "tcp"),
                    ("from_port", 22),
                    ("to_port", 22),
                    ("cidr_ip", "0.0.0.0/0"),
                ]
            ),
            OrderedDict(
                [
                    ("ip_protocol", "tcp"),
                    ("from_port", 80),
                    ("to_port", 80),
                    ("cidr_ip", "0.0.0.0/0"),
                ]
            ),
        ]
        desired_rules = [
            OrderedDict(
                [
                    ("ip_protocol", "tcp"),
                    ("from_port", 22),
                    ("to_port", 22),
                    ("cidr_ip", "0.0.0.0/0"),
                ]
            )
        ]
        # can also use: rules_to_delete = [rule for rule in present_rules if rule not in desired_rules]
        rules_to_delete = [
            OrderedDict(
                [
                    ("ip_protocol", "tcp"),
                    ("from_port", 80),
                    ("to_port", 80),
                    ("cidr_ip", "0.0.0.0/0"),
                ]
            )
        ]
        self.assertEqual(
            boto_secgroup._get_rule_changes(desired_rules, present_rules),
            (rules_to_delete, []),
        )



# -*- coding: utf-8 -*-
"""
    :codeauthor: Andrew Colin Kissa <andrew@topdog.za.net>
"""
from __future__ import absolute_import, print_function, unicode_literals

import salt.states.postgres_initdb as postgres_initdb
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class PostgresInitdbTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.postgres_initdb
    """

    def setup_loader_modules(self):
        return {postgres_initdb: {}}

    def setUp(self):
        """
        Setup data for the tests
        """
        self.name = "/var/lib/psql/data"
        self.ret = {"name": self.name, "changes": {}, "result": False, "comment": ""}
        self.mock_true = MagicMock(return_value=True)
        self.mock_false = MagicMock(return_value=False)

    def tearDown(self):
        del self.ret
        del self.mock_true
        del self.mock_false

    def test_present_existing(self):
        """
        Test existing data directory handled correctly
        """
        with patch.dict(
            postgres_initdb.__salt__, {"postgres.datadir_exists": self.mock_true}
        ):
            _comt = "Postgres data directory {0} is already present".format(self.name)
            self.ret.update({"comment": _comt, "result": True})
            self.assertDictEqual(postgres_initdb.present(self.name), self.ret)

    def test_present_non_existing_pass(self):
        """
        Test non existing data directory ok
        """
        with patch.dict(
            postgres_initdb.__salt__,
            {
                "postgres.datadir_exists": self.mock_false,
                "postgres.datadir_init": self.mock_true,
            },
        ):
            with patch.dict(postgres_initdb.__opts__, {"test": True}):
                _comt = "Postgres data directory {0} is set to be initialized".format(
                    self.name
                )
                self.ret.update({"comment": _comt, "result": None})
                self.assertDictEqual(postgres_initdb.present(self.name), self.ret)

            with patch.dict(postgres_initdb.__opts__, {"test": False}):
                _comt = "Postgres data directory {0} has been initialized".format(
                    self.name
                )
                _changes = {self.name: "Present"}
                self.ret.update({"comment": _comt, "result": True, "changes": _changes})
                self.assertDictEqual(postgres_initdb.present(self.name), self.ret)

    def test_present_non_existing_fail(self):
        """
        Test non existing data directory fail
        """
        with patch.dict(
            postgres_initdb.__salt__,
            {
                "postgres.datadir_exists": self.mock_false,
                "postgres.datadir_init": self.mock_false,
            },
        ):
            with patch.dict(postgres_initdb.__opts__, {"test": False}):
                _comt = "Postgres data directory {0} initialization failed".format(
                    self.name
                )
                self.ret.update({"comment": _comt, "result": False})
                self.assertDictEqual(postgres_initdb.present(self.name), self.ret)



# -*- coding: utf-8 -*-

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.utils.platform
import salt.utils.win_update as win_update

# Import Salt Testing Libs
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase, skipIf


@skipIf(not salt.utils.platform.is_windows(), "System is not Windows")
class WinUpdateTestCase(TestCase):
    """
    Test cases for salt.utils.win_update
    """

    def test_installed_no_updates(self):
        """
        Test installed when there are no updates on the system
        """
        with patch("salt.utils.winapi.Com", autospec=True), patch(
            "win32com.client.Dispatch", autospec=True
        ), patch.object(win_update.WindowsUpdateAgent, "refresh", autospec=True):
            wua = win_update.WindowsUpdateAgent(online=False)
            wua._updates = []

            installed_updates = wua.installed()

            assert installed_updates.updates.Add.call_count == 0

    def test_installed_no_updates_installed(self):
        """
        Test installed when there are no Installed updates on the system
        """
        with patch("salt.utils.winapi.Com", autospec=True), patch(
            "win32com.client.Dispatch", autospec=True
        ), patch.object(win_update.WindowsUpdateAgent, "refresh", autospec=True):
            wua = win_update.WindowsUpdateAgent(online=False)

            wua._updates = [
                MagicMock(IsInstalled=False),
                MagicMock(IsInstalled=False),
                MagicMock(IsInstalled=False),
            ]

            installed_updates = wua.installed()

            assert installed_updates.updates.Add.call_count == 0

    def test_installed_updates_all_installed(self):
        """
        Test installed when all updates on the system are Installed
        """
        with patch("salt.utils.winapi.Com", autospec=True), patch(
            "win32com.client.Dispatch", autospec=True
        ), patch.object(win_update.WindowsUpdateAgent, "refresh", autospec=True):
            wua = win_update.WindowsUpdateAgent(online=False)

            wua._updates = [
                MagicMock(IsInstalled=True),
                MagicMock(IsInstalled=True),
                MagicMock(IsInstalled=True),
            ]

            installed_updates = wua.installed()

            assert installed_updates.updates.Add.call_count == 3

    def test_installed_updates_some_installed(self):
        """
        Test installed when some updates are installed on the system
        """
        with patch("salt.utils.winapi.Com", autospec=True), patch(
            "win32com.client.Dispatch", autospec=True
        ), patch.object(win_update.WindowsUpdateAgent, "refresh", autospec=True):
            wua = win_update.WindowsUpdateAgent(online=False)

            wua._updates = [
                MagicMock(IsInstalled=True),
                MagicMock(IsInstalled=False),
                MagicMock(IsInstalled=True),
                MagicMock(IsInstalled=False),
                MagicMock(IsInstalled=True),
            ]

            installed_updates = wua.installed()

            assert installed_updates.updates.Add.call_count == 3



# -*- coding: utf-8 -*-
"""
    :codeauthor: Jayesh Kariya <jayeshk@saltstack.com>
"""
# Import Python libs
from __future__ import absolute_import, print_function, unicode_literals

# Import Salt Libs
import salt.states.composer as composer
from salt.exceptions import SaltException

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock, patch
from tests.support.unit import TestCase


class ComposerTestCase(TestCase, LoaderModuleMockMixin):
    """
    Test cases for salt.states.composer
    """

    def setup_loader_modules(self):
        return {composer: {}}

    # 'installed' function tests: 1

    def test_installed(self):
        """
        Test to verify that the correct versions of composer
        dependencies are present.
        """
        name = "CURL"

        ret = {"name": name, "result": True, "comment": "", "changes": {}}

        mock = MagicMock(return_value=True)
        with patch.dict(composer.__salt__, {"composer.did_composer_install": mock}):
            comt = "Composer already installed this directory"
            ret.update({"comment": comt})
            self.assertDictEqual(composer.installed(name, always_check=False), ret)

            with patch.dict(composer.__opts__, {"test": True}):
                comt = 'The state of "CURL" will be changed.'
                changes = {
                    "new": "composer install will be run in CURL",
                    "old": "composer install has been run in CURL",
                }
                ret.update({"comment": comt, "result": None, "changes": changes})
                self.assertDictEqual(composer.installed(name), ret)

            with patch.dict(composer.__opts__, {"test": False}):
                mock = MagicMock(side_effect=[SaltException, {}])
                with patch.dict(composer.__salt__, {"composer.install": mock}):
                    comt = "Error executing composer in " "'CURL': "
                    ret.update({"comment": comt, "result": False, "changes": {}})
                    self.assertDictEqual(composer.installed(name), ret)

                    comt = (
                        "Composer install completed successfully,"
                        " output silenced by quiet flag"
                    )
                    ret.update({"comment": comt, "result": True})
                    self.assertDictEqual(composer.installed(name, quiet=True), ret)

    # 'update' function tests: 1

    def test_update(self):
        """
        Test to composer update the directory to ensure we have
        the latest versions of all project dependencies.
        """
        name = "CURL"

        ret = {"name": name, "result": True, "comment": "", "changes": {}}

        changes = {
            "new": "composer install/update will be run in CURL",
            "old": "composer install has not yet been run in CURL",
        }

        mock = MagicMock(return_value=True)
        with patch.dict(composer.__salt__, {"composer.did_composer_install": mock}):
            with patch.dict(composer.__opts__, {"test": True}):
                comt = 'The state of "CURL" will be changed.'
                ret.update({"comment": comt, "result": None, "changes": changes})
                self.assertDictEqual(composer.update(name), ret)

            with patch.dict(composer.__opts__, {"test": False}):
                mock = MagicMock(side_effect=[SaltException, {}])
                with patch.dict(composer.__salt__, {"composer.update": mock}):
                    comt = "Error executing composer in " "'CURL': "
                    ret.update({"comment": comt, "result": False, "changes": {}})
                    self.assertDictEqual(composer.update(name), ret)

                    comt = (
                        "Composer update completed successfully,"
                        " output silenced by quiet flag"
                    )
                    ret.update({"comment": comt, "result": True})
                    self.assertDictEqual(composer.update(name, quiet=True), ret)



# -*- coding: utf-8 -*-

from __future__ import absolute_import

import logging

import salt.modules.slsutil as slsutil
from tests.support.unit import TestCase

log = logging.getLogger(__name__)


class SlsUtilTestCase(TestCase):
    """
    Test cases for salt.modules.slsutil
    """

    def test_banner(self):
        """
        Test banner function
        """
        self.check_banner()
        self.check_banner(width=81)
        self.check_banner(width=20)
        self.check_banner(commentchar="//", borderchar="-")
        self.check_banner(title="title here", text="text here")
        self.check_banner(commentchar=" *")

    def check_banner(
        self,
        width=72,
        commentchar="#",
        borderchar="#",
        blockstart=None,
        blockend=None,
        title=None,
        text=None,
        newline=True,
    ):

        result = slsutil.banner(
            width=width,
            commentchar=commentchar,
            borderchar=borderchar,
            blockstart=blockstart,
            blockend=blockend,
            title=title,
            text=text,
            newline=newline,
        ).splitlines()
        for line in result:
            self.assertEqual(len(line), width)
            self.assertTrue(line.startswith(commentchar))
            self.assertTrue(line.endswith(commentchar.strip()))

    def test_boolstr(self):
        """
        Test boolstr function
        """
        self.assertEqual("yes", slsutil.boolstr(True, true="yes", false="no"))
        self.assertEqual("no", slsutil.boolstr(False, true="yes", false="no"))



# -*- coding: utf-8 -*-
from __future__ import absolute_import, print_function, unicode_literals

import salt.modules.iptables
from tests.support.case import MultimasterModuleCase, MultiMasterTestShellCase
from tests.support.helpers import destructiveTest, skip_if_not_root, slowTest
from tests.support.mixins import AdaptedConfigurationTestCaseMixin
from tests.support.unit import skipIf

HAS_IPTABLES = salt.modules.iptables.__virtual__()
if isinstance(HAS_IPTABLES, tuple):
    HAS_IPTABLES = HAS_IPTABLES[0]


@destructiveTest
@skip_if_not_root
@skipIf(not HAS_IPTABLES, "iptables command is not available")
class TestHandleEvents(
    MultimasterModuleCase, MultiMasterTestShellCase, AdaptedConfigurationTestCaseMixin
):
    """
    Validate the events handling in multimaster environment
    """

    @slowTest
    def test_minion_hangs_on_master_failure_50814(self):
        """
        Check minion handling events for the alive master when another master is dead.
        The case being checked here is described in details in issue #50814.
        """
        disconnect_master_rule = "-i lo -p tcp --dport {0} -j DROP".format(
            self.mm_master_opts["ret_port"]
        )
        # Disconnect the master.
        res = self.run_function(
            "iptables.append",
            ("filter", "INPUT", disconnect_master_rule),
            master_tgt="mm-sub-master",
        )
        # Workaround slow beacons.list_available response
        if not res:
            res = self.run_function(
                "iptables.append",
                ("filter", "INPUT", disconnect_master_rule),
                master_tgt="mm-sub-master",
            )
        self.assertTrue(res)
        try:
            # Send an event. This would return okay.
            res = self.run_function(
                "event.send", ("myco/foo/bar",), master_tgt="mm-sub-master",
            )
            self.assertTrue(res)
            # Send one more event. Minion was hanging on this. This is fixed by #53417
            res = self.run_function(
                "event.send", ("myco/foo/bar",), master_tgt="mm-sub-master", timeout=60,
            )
            self.assertTrue(
                res,
                "Minion is not responding to the second master after the first "
                "one has gone. Check #50814 for details.",
            )
        finally:
            # Remove the firewall rule taking master online back.
            # Since minion could be not responsive now use `salt-call --local` for this.
            res = self.run_call(
                "iptables.delete filter INPUT rule='{0}'".format(
                    disconnect_master_rule
                ),
                local=True,
                timeout=30,
            )
            self.assertEqual(res, ["local:"])
            # Ensure the master is back.
            res = self.run_function(
                "event.send", ("myco/foo/bar",), master_tgt="mm-master",
            )
            self.assertTrue(res)



# -*- coding: utf-8 -*-
"""
    :codeauthor: :email:`Anthony Shaw <anthonyshaw@apache.org>`
"""

# Import Python Libs
from __future__ import absolute_import, print_function, unicode_literals

import tests.support.napalm as napalm_test_support

# Import Salt Testing Libs
from tests.support.mixins import LoaderModuleMockMixin
from tests.support.mock import MagicMock
from tests.support.unit import TestCase

import salt.modules.napalm_users as napalm_users  # NOQA
import salt.modules.napalm_network as napalm_network  # NOQA


class NapalmUsersModuleTestCase(TestCase, LoaderModuleMockMixin):
    def setup_loader_modules(self):
        module_globals = {
            "__salt__": {
                "config.option": MagicMock(
                    return_value={"test": {"driver": "test", "key": "2orgk34kgk34g"}}
                ),
                "file.file_exists": napalm_test_support.true,
                "file.join": napalm_test_support.join,
                "file.get_managed": napalm_test_support.get_managed_file,
                "random.hash": napalm_test_support.random_hash,
                "net.load_template": napalm_network.load_template,
            }
        }

        return {napalm_users: module_globals, napalm_network: module_globals}

    def test_config(self):
        ret = napalm_users.config()
        assert ret["out"] == napalm_test_support.TEST_USERS

    def test_set_users(self):
        ret = napalm_users.set_users({"mircea": {}})
        assert ret["result"] is False

    def test_delete_users(self):
        ret = napalm_users.delete_users({"mircea": {}})
        assert ret["result"] is False



# -*- coding: utf-8 -*-
"""
    tests.multimaster.conftest
    ~~~~~~~~~~~~~~~~~~~~~~~~~~

    Multimaster PyTest prep routines
"""
from __future__ import absolute_import, print_function, unicode_literals

import logging
import os
import shutil

import pytest
import salt.utils.files
from salt.serializers import yaml
from salt.utils.immutabletypes import freeze
from tests.support.pytest.fixtures import *  # pylint: disable=unused-wildcard-import
from tests.support.runtests import RUNTIME_VARS

log = logging.getLogger(__name__)


@pytest.fixture(scope="package")
def salt_mm_master_config(request, salt_factories):
    root_dir = salt_factories._get_root_dir_for_daemon("mm-master")
    with salt.utils.files.fopen(
        os.path.join(RUNTIME_VARS.CONF_DIR, "mm_master")
    ) as rfh:
        config_defaults = yaml.deserialize(rfh.read())

    config_defaults["root_dir"] = root_dir.strpath

    config_overrides = {
        "file_roots": {
            "base": [
                RUNTIME_VARS.TMP_STATE_TREE,
                os.path.join(RUNTIME_VARS.FILES, "file", "base"),
            ],
            # Alternate root to test __env__ choices
            "prod": [
                RUNTIME_VARS.TMP_PRODENV_STATE_TREE,
                os.path.join(RUNTIME_VARS.FILES, "file", "prod"),
            ],
        },
        "pillar_roots": {
            "base": [
                RUNTIME_VARS.TMP_PILLAR_TREE,
                os.path.join(RUNTIME_VARS.FILES, "pillar", "base"),
            ],
            "prod": [RUNTIME_VARS.TMP_PRODENV_PILLAR_TREE],
        },
    }
    return salt_factories.configure_master(
        request,
        "mm-master",
        config_defaults=config_defaults,
        config_overrides=config_overrides,
    )


@pytest.fixture(scope="package")
def salt_mm_minion_config(request, salt_factories, salt_mm_master, salt_mm_sub_master):
    with salt.utils.files.fopen(
        os.path.join(RUNTIME_VARS.CONF_DIR, "mm_minion")
    ) as rfh:
        config_defaults = yaml.deserialize(rfh.read())
    config_defaults["hosts.file"] = os.path.join(RUNTIME_VARS.TMP, "hosts")
    config_defaults["aliases.file"] = os.path.join(RUNTIME_VARS.TMP, "aliases")
    config_defaults["transport"] = request.config.getoption("--transport")

    mm_master_port = salt_mm_master.config["ret_port"]
    mm_sub_master_port = salt_mm_sub_master.config["ret_port"]
    config_overrides = {
        "master_port": "",
        "master": [
            "localhost:{}".format(mm_master_port),
            "localhost:{}".format(mm_sub_master_port),
        ],
        "test.foo": "baz",
    }
    return salt_factories.configure_minion(
        request,
        "mm-minion",
        master_id=salt_mm_master.config["id"],
        config_defaults=config_defaults,
        config_overrides=config_overrides,
    )


@pytest.fixture(scope="package")
def salt_mm_sub_master_config(request, salt_factories, salt_mm_master):
    with salt.utils.files.fopen(
        os.path.join(RUNTIME_VARS.CONF_DIR, "mm_sub_master")
    ) as rfh:
        config_defaults = yaml.deserialize(rfh.read())
    root_dir = salt_factories._get_root_dir_for_daemon("mm-master")

    config_defaults["root_dir"] = root_dir.strpath

    config_overrides = {
        "file_roots": {
            "base": [
                RUNTIME_VARS.TMP_STATE_TREE,
                os.path.join(RUNTIME_VARS.FILES, "file", "base"),
            ],
            # Alternate root to test __env__ choices
            "prod": [
                RUNTIME_VARS.TMP_PRODENV_STATE_TREE,
                os.path.join(RUNTIME_VARS.FILES, "file", "prod"),
            ],
        },
        "pillar_roots": {
            "base": [
                RUNTIME_VARS.TMP_PILLAR_TREE,
                os.path.join(RUNTIME_VARS.FILES, "pillar", "base"),
            ],
            "prod": [RUNTIME_VARS.TMP_PRODENV_PILLAR_TREE],
        },
    }
    return salt_factories.configure_master(
        request,
        "mm-sub-master",
        config_defaults=config_defaults,
        config_overrides=config_overrides,
    )


@pytest.fixture(scope="package")
def salt_mm_sub_minion_config(
    request, salt_factories, salt_mm_master, salt_mm_sub_master
):
    with salt.utils.files.fopen(
        os.path.join(RUNTIME_VARS.CONF_DIR, "mm_sub_minion")
    ) as rfh:
        config_defaults = yaml.deserialize(rfh.read())
    config_defaults["hosts.file"] = os.path.join(RUNTIME_VARS.TMP, "hosts")
    config_defaults["aliases.file"] = os.path.join(RUNTIME_VARS.TMP, "aliases")
    config_defaults["transport"] = request.config.getoption("--transport")

    mm_master_port = salt_mm_master.config["ret_port"]
    mm_sub_master_port = salt_mm_sub_master.config["ret_port"]
    config_overrides = {
        "master_port": "",
        "master": [
            "localhost:{}".format(mm_master_port),
            "localhost:{}".format(mm_sub_master_port),
        ],
        "test.foo": "baz",
    }
    return salt_factories.configure_minion(
        request,
        "mm-sub-minion",
        master_id=salt_mm_sub_master.config["id"],
        config_defaults=config_defaults,
        config_overrides=config_overrides,
    )


@pytest.fixture(scope="package")
def salt_mm_master(request, salt_factories, salt_mm_master_config):
    return salt_factories.spawn_master(request, "mm-master")


@pytest.fixture(scope="package")
def salt_mm_sub_master(
    request, salt_factories, salt_mm_master, salt_mm_sub_master_config
):
    # The secondary salt master depends on the primarily salt master fixture
    # because we need to clone the keys
    for keyfile in ("master.pem", "master.pub"):
        shutil.copyfile(
            os.path.join(salt_mm_master.config["pki_dir"], keyfile),
            os.path.join(salt_mm_sub_master_config["pki_dir"], keyfile),
        )
    return salt_factories.spawn_master(request, "mm-sub-master")


@pytest.fixture(scope="package")
def salt_mm_minion(
    request, salt_factories, salt_mm_master, salt_mm_sub_master, salt_mm_minion_config
):
    return salt_factories.spawn_minion(
        request, "mm-minion", master_id=salt_mm_master.config["id"]
    )


@pytest.fixture(scope="package")
def salt_mm_sub_minion(
    request,
    salt_factories,
    salt_mm_master,
    salt_mm_sub_master,
    salt_mm_sub_minion_config,
):
    return salt_factories.spawn_minion(
        request, "mm-sub-minion", master_id=salt_mm_sub_master.config["id"]
    )


@pytest.fixture(scope="package", autouse=True)
def bridge_pytest_and_runtests(  # pylint: disable=function-redefined
    reap_stray_processes,
    base_env_state_tree_root_dir,
    prod_env_state_tree_root_dir,
    base_env_pillar_tree_root_dir,
    prod_env_pillar_tree_root_dir,
    salt_factories,
    salt_mm_master,
    salt_mm_minion,
    salt_mm_sub_master,
    salt_mm_sub_minion,
):
    # Make sure unittest2 uses the pytest generated configuration
    RUNTIME_VARS.RUNTIME_CONFIGS["mm_master"] = freeze(salt_mm_master.config)
    RUNTIME_VARS.RUNTIME_CONFIGS["mm_minion"] = freeze(salt_mm_minion.config)
    RUNTIME_VARS.RUNTIME_CONFIGS["mm_sub_master"] = freeze(salt_mm_sub_master.config)
    RUNTIME_VARS.RUNTIME_CONFIGS["mm_sub_minion"] = freeze(salt_mm_sub_minion.config)

    # Make sure unittest2 classes know their paths
    RUNTIME_VARS.TMP_MM_CONF_DIR = os.path.dirname(salt_mm_master.config["conf_file"])
    RUNTIME_VARS.TMP_MM_MINION_CONF_DIR = os.path.dirname(
        salt_mm_minion.config["conf_file"]
    )
    RUNTIME_VARS.TMP_MM_SUB_CONF_DIR = os.path.dirname(
        salt_mm_sub_master.config["conf_file"]
    )
    RUNTIME_VARS.TMP_MM_SUB_MINION_CONF_DIR = os.path.dirname(
        salt_mm_sub_minion.config["conf_file"]
    )
